@@ -1,66 +1,64 @@
 """Remote datastore."""
-# pylint: disable=missing-type-doc
-import logging
-
+from pymodbus.datastore import ModbusBaseSlaveContext
 from pymodbus.exceptions import NotImplementedException
-from pymodbus.interfaces import IModbusSlaveContext
-
-
-# ---------------------------------------------------------------------------#
-#  Logging
-# ---------------------------------------------------------------------------#
-_logger = logging.getLogger(__name__)
+from pymodbus.logging import Log
 
 
 # ---------------------------------------------------------------------------#
 #  Context
 # ---------------------------------------------------------------------------#
-class RemoteSlaveContext(IModbusSlaveContext):
+class RemoteSlaveContext(ModbusBaseSlaveContext):
     """TODO.
 
     This creates a modbus data model that connects to
     a remote device (depending on the client used)
     """
 
-    def __init__(self, client, unit=None):
+    def __init__(self, client, slave=None):
         """Initialize the datastores.
 
         :param client: The client to retrieve values with
-        :param unit: Unit ID of the remote slave
+        :param slave: Unit ID of the remote slave
         """
         self._client = client
-        self.unit = unit
+        self.slave = slave
         self.result = None
         self.__build_mapping()
         if not self.__set_callbacks:
-            _logger.error("Init went wrong.")
+            Log.error("Init went wrong.")
 
     def reset(self):
         """Reset all the datastores to their default values."""
         raise NotImplementedException()
 
-    def validate(self, fc_as_hex, address, count=1):
+    def validate(self, _fc_as_hex, _address, _count):
         """Validate the request to make sure it is in range.
 
-        :param fc_as_hex: The function we are working with
-        :param address: The starting address
-        :param count: The number of values to test
-        :returns: True if the request in within range, False otherwise
+        :returns: True
         """
-        txt = f"validate[{fc_as_hex}] {address}:{count}"
-        _logger.debug(txt)
-        self.result = self.__get_callbacks[self.decode(fc_as_hex)](address, count)
-        return not self.result.isError()
+        return True
 
-    def getValues(self, fc_as_hex, address, count=1):
-        """Get values from real call in validate"""
+    def getValues(self, fc_as_hex, _address, _count=1):
+        """Get values from real call in validate."""
+        if fc_as_hex in self._write_fc:
+            return [0]
+        group_fx = self.decode(fc_as_hex)
+        func_fc = self.__get_callbacks[group_fx]
+        self.result = func_fc(_address, _count)
         return self.__extract_result(self.decode(fc_as_hex), self.result)
 
     def setValues(self, fc_as_hex, address, values):
-        """Set the datastore with the supplied values.
-
-        Already done in validate
-        """
+        """Set the datastore with the supplied values."""
+        group_fx = self.decode(fc_as_hex)
+        if fc_as_hex in self._write_fc:
+            func_fc = self.__set_callbacks[f"{group_fx}{fc_as_hex}"]
+            if fc_as_hex in {0x0F, 0x10}:
+                self.result = func_fc(address, values)
+            else:
+                self.result = func_fc(address, values[0])
+        if self.result.isError():
+            return self.result
+        return None
 
     def __str__(self):
         """Return a string representation of the context.
@@ -72,8 +70,8 @@ class RemoteSlaveContext(IModbusSlaveContext):
     def __build_mapping(self):
         """Build the function code mapper."""
         kwargs = {}
-        if self.unit:
-            kwargs["unit"] = self.unit
+        if self.slave:
+            kwargs["slave"] = self.slave
         self.__get_callbacks = {
             "d": lambda a, c: self._client.read_discrete_inputs(  # pylint: disable=unnecessary-lambda
                 a, c, **kwargs
@@ -89,19 +87,32 @@ class RemoteSlaveContext(IModbusSlaveContext):
             ),
         }
         self.__set_callbacks = {
-            "d": lambda a, v: self._client.write_coils(  # pylint: disable=unnecessary-lambda
+            "d5": lambda a, v: self._client.write_coil(  # pylint: disable=unnecessary-lambda
                 a, v, **kwargs
             ),
-            "c": lambda a, v: self._client.write_coils(  # pylint: disable=unnecessary-lambda
+            "d15": lambda a, v: self._client.write_coils(  # pylint: disable=unnecessary-lambda
                 a, v, **kwargs
             ),
-            "h": lambda a, v: self._client.write_registers(  # pylint: disable=unnecessary-lambda
+            "c5": lambda a, v: self._client.write_coil(  # pylint: disable=unnecessary-lambda
                 a, v, **kwargs
             ),
-            "i": lambda a, v: self._client.write_registers(  # pylint: disable=unnecessary-lambda
+            "c15": lambda a, v: self._client.write_coils(  # pylint: disable=unnecessary-lambda
+                a, v, **kwargs
+            ),
+            "h6": lambda a, v: self._client.write_register(  # pylint: disable=unnecessary-lambda
+                a, v, **kwargs
+            ),
+            "h16": lambda a, v: self._client.write_registers(  # pylint: disable=unnecessary-lambda
+                a, v, **kwargs
+            ),
+            "i6": lambda a, v: self._client.write_register(  # pylint: disable=unnecessary-lambda
+                a, v, **kwargs
+            ),
+            "i16": lambda a, v: self._client.write_registers(  # pylint: disable=unnecessary-lambda
                 a, v, **kwargs
             ),
         }
+        self._write_fc = (0x05, 0x06, 0x0F, 0x10)
 
     def __extract_result(self, fc_as_hex, result):
         """Extract the values out of a response.
