@@ -4,12 +4,22 @@ These are the device management handlers.  They should be
 maintained in the server context and the various methods
 should be inserted in the correct locations.
 """
-# pylint: disable=missing-type-doc
-from collections import OrderedDict
+from __future__ import annotations
+
+
+__all__ = [
+    "ModbusPlusStatistics",
+    "ModbusDeviceIdentification",
+    "DeviceInformationFactory",
+]
+
 import struct
 
-from pymodbus.constants import DeviceInformation
-from pymodbus.interfaces import Singleton
+# pylint: disable=missing-type-doc
+from collections import OrderedDict
+
+from pymodbus.constants import INTERNAL_ERROR, DeviceInformation
+from pymodbus.events import ModbusEvent
 from pymodbus.utilities import dict_property
 
 
@@ -211,26 +221,18 @@ class ModbusDeviceIdentification:
     # -------------------------------------------------------------------------#
     #  Properties
     # -------------------------------------------------------------------------#
-    VendorName = dict_property(
-        lambda s: s.__data, 0  # pylint: disable=protected-access
-    )
-    ProductCode = dict_property(
-        lambda s: s.__data, 1  # pylint: disable=protected-access
-    )
-    MajorMinorRevision = dict_property(
-        lambda s: s.__data, 2  # pylint: disable=protected-access
-    )
+    # fmt: off
+    VendorName = dict_property(lambda s: s.__data, 0)  # pylint: disable=protected-access
+    ProductCode = dict_property(lambda s: s.__data, 1)  # pylint: disable=protected-access
+    MajorMinorRevision = dict_property(lambda s: s.__data, 2)  # pylint: disable=protected-access
     VendorUrl = dict_property(lambda s: s.__data, 3)  # pylint: disable=protected-access
-    ProductName = dict_property(
-        lambda s: s.__data, 4  # pylint: disable=protected-access
-    )
+    ProductName = dict_property(lambda s: s.__data, 4)  # pylint: disable=protected-access
     ModelName = dict_property(lambda s: s.__data, 5)  # pylint: disable=protected-access
-    UserApplicationName = dict_property(
-        lambda s: s.__data, 6  # pylint: disable=protected-access
-    )
+    UserApplicationName = dict_property(lambda s: s.__data, 6)  # pylint: disable=protected-access
+    # fmt: on
 
 
-class DeviceInformationFactory(Singleton):  # pylint: disable=too-few-public-methods
+class DeviceInformationFactory:  # pylint: disable=too-few-public-methods
     """This is a helper factory.
 
     That really just hides
@@ -239,28 +241,28 @@ class DeviceInformationFactory(Singleton):  # pylint: disable=too-few-public-met
     """
 
     __lookup = {
-        DeviceInformation.Basic: lambda c, r, i: c.__gets(  # pylint: disable=protected-access
+        DeviceInformation.BASIC: lambda c, r, i: c.__gets(  # pylint: disable=protected-access
             r, list(range(i, 0x03))
         ),
-        DeviceInformation.Regular: lambda c, r, i: c.__gets(  # pylint: disable=protected-access
+        DeviceInformation.REGULAR: lambda c, r, i: c.__gets(  # pylint: disable=protected-access
             r,
             list(range(i, 0x07))
             if c.__get(r, i)[i]  # pylint: disable=protected-access
             else list(range(0, 0x07)),
         ),
-        DeviceInformation.Extended: lambda c, r, i: c.__gets(  # pylint: disable=protected-access
+        DeviceInformation.EXTENDED: lambda c, r, i: c.__gets(  # pylint: disable=protected-access
             r,
             [x for x in range(i, 0x100) if x not in range(0x07, 0x80)]
             if c.__get(r, i)[i]  # pylint: disable=protected-access
             else [x for x in range(0, 0x100) if x not in range(0x07, 0x80)],
         ),
-        DeviceInformation.Specific: lambda c, r, i: c.__get(  # pylint: disable=protected-access
+        DeviceInformation.SPECIFIC: lambda c, r, i: c.__get(  # pylint: disable=protected-access
             r, i
         ),
     }
 
     @classmethod
-    def get(cls, control, read_code=DeviceInformation.Basic, object_id=0x00):
+    def get(cls, control, read_code=DeviceInformation.BASIC, object_id=0x00):
         """Get the requested device data from the system.
 
         :param control: The control block to pull data from
@@ -291,6 +293,10 @@ class DeviceInformationFactory(Singleton):  # pylint: disable=too-few-public-met
         """
         return {oid: identity[oid] for oid in object_ids if identity[oid]}
 
+    def __init__(self):
+        """Prohibit objects."""
+        raise RuntimeError(INTERNAL_ERROR)
+
 
 # ---------------------------------------------------------------------------#
 #  Counters Handler
@@ -415,33 +421,23 @@ class ModbusCountersHandler:
     # -------------------------------------------------------------------------#
     #  Properties
     # -------------------------------------------------------------------------#
-    BusMessage = dict_property(
-        lambda s: s.__data, 0  # pylint: disable=protected-access
-    )
-    BusCommunicationError = dict_property(
-        lambda s: s.__data, 1  # pylint: disable=protected-access
-    )
-    BusExceptionError = dict_property(
-        lambda s: s.__data, 2  # pylint: disable=protected-access
-    )
-    SlaveMessage = dict_property(
-        lambda s: s.__data, 3  # pylint: disable=protected-access
-    )
-    SlaveNoResponse = dict_property(
-        lambda s: s.__data, 4  # pylint: disable=protected-access
-    )
+    # fmt: off
+    BusMessage = dict_property(lambda s: s.__data, 0)  # pylint: disable=protected-access
+    BusCommunicationError = dict_property(lambda s: s.__data, 1)  # pylint: disable=protected-access
+    BusExceptionError = dict_property(lambda s: s.__data, 2)  # pylint: disable=protected-access
+    SlaveMessage = dict_property(lambda s: s.__data, 3)  # pylint: disable=protected-access
+    SlaveNoResponse = dict_property(lambda s: s.__data, 4)  # pylint: disable=protected-access
     SlaveNAK = dict_property(lambda s: s.__data, 5)  # pylint: disable=protected-access
     SlaveBusy = dict_property(lambda s: s.__data, 6)  # pylint: disable=protected-access
-    BusCharacterOverrun = dict_property(
-        lambda s: s.__data, 7  # pylint: disable=protected-access
-    )
+    BusCharacterOverrun = dict_property(lambda s: s.__data, 7)  # pylint: disable=protected-access
     Event = dict_property(lambda s: s.__data, 8)  # pylint: disable=protected-access
+    # fmt: on
 
 
 # ---------------------------------------------------------------------------#
 #  Main server control block
 # ---------------------------------------------------------------------------#
-class ModbusControlBlock(Singleton):
+class ModbusControlBlock:
     """This is a global singleton that controls all system information.
 
     All activity should be logged here and all diagnostic requests
@@ -450,13 +446,12 @@ class ModbusControlBlock(Singleton):
 
     __mode = "ASCII"
     __diagnostic = [False] * 16
-    __instance = None
     __listen_only = False
-    __delimiter = "\r"
+    __delimiter = b"\r"
     __counters = ModbusCountersHandler()
     __identity = ModbusDeviceIdentification()
     __plus = ModbusPlusStatistics()
-    __events = []
+    __events: list[ModbusEvent] = []
 
     # -------------------------------------------------------------------------#
     #  Magic
@@ -475,10 +470,16 @@ class ModbusControlBlock(Singleton):
         """
         return self.__counters.__iter__()
 
+    def __new__(cls, *_args, **_kwargs):
+        """Create a new instance."""
+        if "_inst" not in vars(cls):
+            cls._inst = object.__new__(cls)
+        return cls._inst
+
     # -------------------------------------------------------------------------#
     #  Events
     # -------------------------------------------------------------------------#
-    def addEvent(self, event):  # pylint: disable=invalid-name
+    def addEvent(self, event: ModbusEvent):
         """Add a new event to the event log.
 
         :param event: A new event to add to the log
@@ -487,7 +488,7 @@ class ModbusControlBlock(Singleton):
         self.__events = self.__events[0:64]  # chomp to 64 entries
         self.Counter.Event += 1
 
-    def getEvents(self):  # pylint: disable=invalid-name
+    def getEvents(self):
         """Return an encoded collection of the event log.
 
         :returns: The encoded events packet
@@ -495,7 +496,7 @@ class ModbusControlBlock(Singleton):
         events = [event.encode() for event in self.__events]
         return b"".join(events)
 
-    def clearEvents(self):  # pylint: disable=invalid-name
+    def clearEvents(self):
         """Clear the current list of events."""
         self.__events = []
 
@@ -516,7 +517,7 @@ class ModbusControlBlock(Singleton):
     # -------------------------------------------------------------------------#
     #  Listen Properties
     # -------------------------------------------------------------------------#
-    def _setListenOnly(self, value):  # pylint: disable=invalid-name
+    def _setListenOnly(self, value):
         """Toggle the listen only status.
 
         :param value: The value to set the listen status to
@@ -528,7 +529,7 @@ class ModbusControlBlock(Singleton):
     # -------------------------------------------------------------------------#
     #  Mode Properties
     # -------------------------------------------------------------------------#
-    def _setMode(self, mode):  # pylint: disable=invalid-name
+    def _setMode(self, mode):
         """Toggle the current serial mode.
 
         :param mode: The data transfer method in (RTU, ASCII)
@@ -541,7 +542,7 @@ class ModbusControlBlock(Singleton):
     # -------------------------------------------------------------------------#
     #  Delimiter Properties
     # -------------------------------------------------------------------------#
-    def _setDelimiter(self, char):  # pylint: disable=invalid-name
+    def _setDelimiter(self, char):
         """Change the serial delimiter character.
 
         :param char: The new serial delimiter character
@@ -560,7 +561,7 @@ class ModbusControlBlock(Singleton):
     # -------------------------------------------------------------------------#
     #  Diagnostic Properties
     # -------------------------------------------------------------------------#
-    def setDiagnostic(self, mapping):  # pylint: disable=invalid-name
+    def setDiagnostic(self, mapping):
         """Set the value in the diagnostic register.
 
         :param mapping: Dictionary of key:value pairs to set
@@ -569,7 +570,7 @@ class ModbusControlBlock(Singleton):
             if entry[0] >= 0 and entry[0] < len(self.__diagnostic):
                 self.__diagnostic[entry[0]] = bool(entry[1])
 
-    def getDiagnostic(self, bit):  # pylint: disable=invalid-name
+    def getDiagnostic(self, bit):
         """Get the value in the diagnostic register.
 
         :param bit: The bit to get
@@ -582,20 +583,9 @@ class ModbusControlBlock(Singleton):
             return None
         return None
 
-    def getDiagnosticRegister(self):  # pylint: disable=invalid-name
+    def getDiagnosticRegister(self):
         """Get the entire diagnostic register.
 
         :returns: The diagnostic register collection
         """
         return self.__diagnostic
-
-
-# ---------------------------------------------------------------------------#
-#  Exported Identifiers
-# ---------------------------------------------------------------------------#
-__all__ = [
-    "ModbusPlusStatistics",
-    "ModbusDeviceIdentification",
-    "DeviceInformationFactory",
-    "ModbusControlBlock",
-]
