@@ -3,12 +3,52 @@
 These need to be tied into a the current server context
 or linked to the appropriate data
 """
+
+__all__ = [
+    "DiagnosticStatusRequest",
+    "DiagnosticStatusResponse",
+    "ReturnQueryDataRequest",
+    "ReturnQueryDataResponse",
+    "RestartCommunicationsOptionRequest",
+    "RestartCommunicationsOptionResponse",
+    "ReturnDiagnosticRegisterRequest",
+    "ReturnDiagnosticRegisterResponse",
+    "ChangeAsciiInputDelimiterRequest",
+    "ChangeAsciiInputDelimiterResponse",
+    "ForceListenOnlyModeRequest",
+    "ForceListenOnlyModeResponse",
+    "ClearCountersRequest",
+    "ClearCountersResponse",
+    "ReturnBusMessageCountRequest",
+    "ReturnBusMessageCountResponse",
+    "ReturnBusCommunicationErrorCountRequest",
+    "ReturnBusCommunicationErrorCountResponse",
+    "ReturnBusExceptionErrorCountRequest",
+    "ReturnBusExceptionErrorCountResponse",
+    "ReturnSlaveMessageCountRequest",
+    "ReturnSlaveMessageCountResponse",
+    "ReturnSlaveNoResponseCountRequest",
+    "ReturnSlaveNoResponseCountResponse",
+    "ReturnSlaveNAKCountRequest",
+    "ReturnSlaveNAKCountResponse",
+    "ReturnSlaveBusyCountRequest",
+    "ReturnSlaveBusyCountResponse",
+    "ReturnSlaveBusCharacterOverrunCountRequest",
+    "ReturnSlaveBusCharacterOverrunCountResponse",
+    "ReturnIopOverrunCountRequest",
+    "ReturnIopOverrunCountResponse",
+    "ClearOverrunCountRequest",
+    "ClearOverrunCountResponse",
+    "GetClearModbusPlusRequest",
+    "GetClearModbusPlusResponse",
+]
+
 # pylint: disable=missing-type-doc
 import struct
 
 from pymodbus.constants import ModbusPlusOperation, ModbusStatus
 from pymodbus.device import ModbusControlBlock
-from pymodbus.exceptions import NotImplementedException
+from pymodbus.exceptions import ModbusException, NotImplementedException
 from pymodbus.pdu import ModbusRequest, ModbusResponse
 from pymodbus.utilities import pack_bitstring
 
@@ -26,6 +66,7 @@ class DiagnosticStatusRequest(ModbusRequest):
     """This is a base class for all of the diagnostic request functions."""
 
     function_code = 0x08
+    function_code_name = "diagnostic_status"
     _rtu_frame_size = 8
 
     def __init__(self, **kwargs):
@@ -46,7 +87,7 @@ class DiagnosticStatusRequest(ModbusRequest):
                 packet += self.message.encode()
             elif isinstance(self.message, bytes):
                 packet += self.message
-            elif isinstance(self.message, list):
+            elif isinstance(self.message, (list, tuple)):
                 for piece in self.message:
                     packet += struct.pack(">H", piece)
             elif isinstance(self.message, int):
@@ -60,8 +101,11 @@ class DiagnosticStatusRequest(ModbusRequest):
         """
         (
             self.sub_function_code,  # pylint: disable=attribute-defined-outside-init
-            self.message,
-        ) = struct.unpack(">HH", data)
+        ) = struct.unpack(">H", data[:2])
+        if self.sub_function_code == ReturnQueryDataRequest.sub_function_code:
+            self.message = data[2:]
+        else:
+            (self.message,) = struct.unpack(">H", data[2:])
 
     def get_response_pdu_size(self):
         """Get response pdu size.
@@ -105,7 +149,7 @@ class DiagnosticStatusResponse(ModbusResponse):
                 packet += self.message.encode()
             elif isinstance(self.message, bytes):
                 packet += self.message
-            elif isinstance(self.message, list):
+            elif isinstance(self.message, (list, tuple)):
                 for piece in self.message:
                     packet += struct.pack(">H", piece)
             elif isinstance(self.message, int):
@@ -117,18 +161,19 @@ class DiagnosticStatusResponse(ModbusResponse):
 
         :param data: The data to decode into the function code
         """
-        word_len = len(data) // 2
-        if len(data) % 2:
-            word_len += 1
-            data = data + b"0"
-        data = struct.unpack(">" + "H" * word_len, data)
         (
             self.sub_function_code,  # pylint: disable=attribute-defined-outside-init
-            self.message,
-        ) = (
-            data[0],
-            data[1:],
-        )
+        ) = struct.unpack(">H", data[:2])
+        data = data[2:]
+        if self.sub_function_code == ReturnQueryDataRequest.sub_function_code:
+            self.message = data
+        else:
+            word_len = len(data) // 2
+            if len(data) % 2:
+                word_len += 1
+                data += b"0"
+            data = struct.unpack(">" + "H" * word_len, data)
+            self.message = data
 
 
 class DiagnosticStatusSimpleRequest(DiagnosticStatusRequest):
@@ -144,7 +189,7 @@ class DiagnosticStatusSimpleRequest(DiagnosticStatusRequest):
     """
 
     def __init__(self, data=0x0000, **kwargs):
-        """Initialize a simple diagnostic request
+        """Initialize a simple diagnostic request.
 
         The data defaults to 0x0000 if not provided as over half
         of the functions require it.
@@ -190,18 +235,17 @@ class ReturnQueryDataRequest(DiagnosticStatusRequest):
 
     sub_function_code = 0x0000
 
-    def __init__(self, message=0x0000, unit=None, **kwargs):
+    def __init__(self, message=b"\x00\x00", slave=None, **kwargs):
         """Initialize a new instance of the request.
 
         :param message: The message to send to loopback
         """
-        DiagnosticStatusRequest.__init__(self, unit=unit, **kwargs)
-        if isinstance(message, list):
-            self.message = message
-        else:
-            self.message = [message]
+        DiagnosticStatusRequest.__init__(self, slave=slave, **kwargs)
+        if not isinstance(message, bytes):
+            raise ModbusException(f"message({type(message)}) must be bytes")
+        self.message = message
 
-    def execute(self, *args):  # pylint: disable=unused-argument
+    def execute(self, *_args):
         """Execute the loopback request (builds the response).
 
         :returns: The populated loopback response message
@@ -219,16 +263,15 @@ class ReturnQueryDataResponse(DiagnosticStatusResponse):
 
     sub_function_code = 0x0000
 
-    def __init__(self, message=0x0000, **kwargs):
+    def __init__(self, message=b"\x00\x00", **kwargs):
         """Initialize a new instance of the response.
 
         :param message: The message to loopback
         """
         DiagnosticStatusResponse.__init__(self, **kwargs)
-        if isinstance(message, list):
-            self.message = message
-        else:
-            self.message = [message]
+        if not isinstance(message, bytes):
+            raise ModbusException(f"message({type(message)}) must be bytes")
+        self.message = message
 
 
 # ---------------------------------------------------------------------------#
@@ -247,18 +290,18 @@ class RestartCommunicationsOptionRequest(DiagnosticStatusRequest):
 
     sub_function_code = 0x0001
 
-    def __init__(self, toggle=False, unit=None, **kwargs):
+    def __init__(self, toggle=False, slave=None, **kwargs):
         """Initialize a new request.
 
         :param toggle: Set to True to toggle, False otherwise
         """
-        DiagnosticStatusRequest.__init__(self, unit=unit, **kwargs)
+        DiagnosticStatusRequest.__init__(self, slave=slave, **kwargs)
         if toggle:
-            self.message = [ModbusStatus.On]
+            self.message = [ModbusStatus.ON]
         else:
-            self.message = [ModbusStatus.Off]
+            self.message = [ModbusStatus.OFF]
 
-    def execute(self, *args):  # pylint: disable=unused-argument
+    def execute(self, *_args):
         """Clear event log and restart.
 
         :returns: The initialized response message
@@ -287,16 +330,16 @@ class RestartCommunicationsOptionResponse(DiagnosticStatusResponse):
         """
         DiagnosticStatusResponse.__init__(self, **kwargs)
         if toggle:
-            self.message = [ModbusStatus.On]
+            self.message = [ModbusStatus.ON]
         else:
-            self.message = [ModbusStatus.Off]
+            self.message = [ModbusStatus.OFF]
 
 
 # ---------------------------------------------------------------------------#
 #  Diagnostic Sub Code 02
 # ---------------------------------------------------------------------------#
 class ReturnDiagnosticRegisterRequest(DiagnosticStatusSimpleRequest):
-    """The contents of the remote device"s 16-bit diagnostic register are returned in the response."""
+    """The contents of the remote device's 16-bit diagnostic register are returned in the response."""
 
     sub_function_code = 0x0002
 
@@ -313,7 +356,7 @@ class ReturnDiagnosticRegisterRequest(DiagnosticStatusSimpleRequest):
 class ReturnDiagnosticRegisterResponse(DiagnosticStatusSimpleResponse):
     """Return diagnostic register.
 
-    The contents of the remote device"s 16-bit diagnostic register are
+    The contents of the remote device's 16-bit diagnostic register are
     returned in the response
     """
 
@@ -577,10 +620,10 @@ class ReturnSlaveNoResponseCountRequest(DiagnosticStatusSimpleRequest):
         :returns: The initialized response message
         """
         count = _MCB.Counter.SlaveNoResponse
-        return ReturnSlaveNoReponseCountResponse(count)
+        return ReturnSlaveNoResponseCountResponse(count)
 
 
-class ReturnSlaveNoReponseCountResponse(DiagnosticStatusSimpleResponse):
+class ReturnSlaveNoResponseCountResponse(DiagnosticStatusSimpleResponse):
     """Return slave no response.
 
     The response data field returns the quantity of messages addressed to the
@@ -773,9 +816,9 @@ class GetClearModbusPlusRequest(DiagnosticStatusSimpleRequest):
 
     sub_function_code = 0x0015
 
-    def __init__(self, unit=None, **kwargs):
+    def __init__(self, slave=None, **kwargs):
         """Initialize."""
-        super().__init__(unit=unit, **kwargs)
+        super().__init__(slave=slave, **kwargs)
 
     def get_response_pdu_size(self):
         """Return a series of 54 16-bit words (108 bytes) in the data field of the response.
@@ -785,7 +828,7 @@ class GetClearModbusPlusRequest(DiagnosticStatusSimpleRequest):
         Func_code (1 byte) + Sub function code (2 byte) + Operation (2 byte) + Data (108 bytes)
         :return:
         """
-        if self.message == ModbusPlusOperation.GetStatistics:
+        if self.message == ModbusPlusOperation.GET_STATISTICS:
             data = 2 + 108  # byte count(2) + data (54*2)
         else:
             data = 0
@@ -797,7 +840,7 @@ class GetClearModbusPlusRequest(DiagnosticStatusSimpleRequest):
         :returns: The initialized response message
         """
         message = None  # the clear operation does not return info
-        if self.message == ModbusPlusOperation.ClearStatistics:
+        if self.message == ModbusPlusOperation.CLEAR_STATISTICS:
             _MCB.Plus.reset()
             message = self.message
         else:
@@ -825,46 +868,3 @@ class GetClearModbusPlusResponse(DiagnosticStatusSimpleResponse):
     """
 
     sub_function_code = 0x0015
-
-
-# ---------------------------------------------------------------------------#
-#  Exported symbols
-# ---------------------------------------------------------------------------#
-__all__ = [
-    "DiagnosticStatusRequest",
-    "DiagnosticStatusResponse",
-    "ReturnQueryDataRequest",
-    "ReturnQueryDataResponse",
-    "RestartCommunicationsOptionRequest",
-    "RestartCommunicationsOptionResponse",
-    "ReturnDiagnosticRegisterRequest",
-    "ReturnDiagnosticRegisterResponse",
-    "ChangeAsciiInputDelimiterRequest",
-    "ChangeAsciiInputDelimiterResponse",
-    "ForceListenOnlyModeRequest",
-    "ForceListenOnlyModeResponse",
-    "ClearCountersRequest",
-    "ClearCountersResponse",
-    "ReturnBusMessageCountRequest",
-    "ReturnBusMessageCountResponse",
-    "ReturnBusCommunicationErrorCountRequest",
-    "ReturnBusCommunicationErrorCountResponse",
-    "ReturnBusExceptionErrorCountRequest",
-    "ReturnBusExceptionErrorCountResponse",
-    "ReturnSlaveMessageCountRequest",
-    "ReturnSlaveMessageCountResponse",
-    "ReturnSlaveNoResponseCountRequest",
-    "ReturnSlaveNoReponseCountResponse",
-    "ReturnSlaveNAKCountRequest",
-    "ReturnSlaveNAKCountResponse",
-    "ReturnSlaveBusyCountRequest",
-    "ReturnSlaveBusyCountResponse",
-    "ReturnSlaveBusCharacterOverrunCountRequest",
-    "ReturnSlaveBusCharacterOverrunCountResponse",
-    "ReturnIopOverrunCountRequest",
-    "ReturnIopOverrunCountResponse",
-    "ClearOverrunCountRequest",
-    "ClearOverrunCountResponse",
-    "GetClearModbusPlusRequest",
-    "GetClearModbusPlusResponse",
-]
