@@ -1,21 +1,16 @@
 """Ascii_framer."""
 # pylint: disable=missing-type-doc
-from binascii import a2b_hex, b2a_hex
-import logging
 import struct
+from binascii import a2b_hex, b2a_hex
 
 from pymodbus.exceptions import ModbusIOException
-from pymodbus.framer import BYTE_ORDER, FRAME_HEADER, ModbusFramer
+from pymodbus.framer.base import BYTE_ORDER, FRAME_HEADER, ModbusFramer
+from pymodbus.logging import Log
 from pymodbus.utilities import checkLRC, computeLRC
 
 
 ASCII_FRAME_HEADER = BYTE_ORDER + FRAME_HEADER
 
-# --------------------------------------------------------------------------- #
-# Logging
-# --------------------------------------------------------------------------- #
-_logger = logging.getLogger(__name__)
-
 
 # --------------------------------------------------------------------------- #
 # Modbus ASCII Message
@@ -42,23 +37,23 @@ class ModbusAsciiFramer(ModbusFramer):
 
         :param decoder: The decoder implementation to use
         """
-        self._buffer = b""
-        self._header = {"lrc": "0000", "len": 0, "uid": 0x00}
+        super().__init__(decoder, client)
         self._hsize = 0x02
         self._start = b":"
         self._end = b"\r\n"
-        self.decoder = decoder
-        self.client = client
 
     # ----------------------------------------------------------------------- #
     # Private Helper Functions
     # ----------------------------------------------------------------------- #
+    def _process(self, callback, error=False):
+        """Process incoming packets irrespective error condition."""
+
     def decode_data(self, data):
         """Decode data."""
         if len(data) > 1:
             uid = int(data[1:3], 16)
             fcode = int(data[3:5], 16)
-            return {"unit": uid, "fcode": fcode}
+            return {"slave": uid, "fcode": fcode}
         return {}
 
     def checkFrame(self):
@@ -101,16 +96,6 @@ class ModbusAsciiFramer(ModbusFramer):
         """
         return len(self._buffer) > 1
 
-    def addToFrame(self, message):
-        """Add the next message to the frame buffer.
-
-        This should be used before the decoding while loop to add the received
-        data to the buffer handle.
-
-        :param message: The most recent packet
-        """
-        self._buffer += message
-
     def getFrame(self):
         """Get the next frame from the buffer.
 
@@ -123,72 +108,26 @@ class ModbusAsciiFramer(ModbusFramer):
             return a2b_hex(buffer)
         return b""
 
-    def resetFrame(self):  # pylint: disable=invalid-name
-        """Reset the entire message frame.
-
-        This allows us to skip ovver errors that may be in the stream.
-        It is hard to know if we are simply out of sync or if there is
-        an error in the stream as we have no way to check the start or
-        end of the message (python just doesn"t have the resolution to
-        check for millisecond delays).
-        """
-        self._buffer = b""
-        self._header = {"lrc": "0000", "len": 0, "uid": 0x00}
-
-    def populateResult(self, result):
-        """Populate the modbus result header.
-
-        The serial packets do not have any header information
-        that is copied.
-
-        :param result: The response packet
-        """
-        result.unit_id = self._header["uid"]
-
     # ----------------------------------------------------------------------- #
     # Public Member Functions
     # ----------------------------------------------------------------------- #
-    def processIncomingPacket(
-        self, data, callback, unit, **kwargs
-    ):  # pylint: disable=arguments-differ
-        """Process new packet pattern.
-
-        This takes in a new request packet, adds it to the current
-        packet stream, and performs framing on it. That is, checks
-        for complete messages, and once found, will process all that
-        exist.  This handles the case when we read N + 1 or 1 // N
-        messages at a time instead of 1.
-
-        The processed and decoded messages are pushed to the callback
-        function to process and send.
-
-        :param data: The new packet data
-        :param callback: The function to send results to
-        :param unit: Process if unit id matches, ignore otherwise (could be a
-               list of unit ids (server) or single unit id(client/server))
-        :param kwargs:
-        :raises ModbusIOException:
-        """
-        if not isinstance(unit, (list, tuple)):
-            unit = [unit]
-        single = kwargs.get("single", False)
-        self.addToFrame(data)
+    def frameProcessIncomingPacket(self, single, callback, slave, _tid=None, **kwargs):
+        """Process new packet pattern."""
         while self.isFrameReady():
-            if self.checkFrame():
-                if self._validate_unit_id(unit, single):
-                    frame = self.getFrame()
-                    if (result := self.decoder.decode(frame)) is None:
-                        raise ModbusIOException("Unable to decode response")
-                    self.populateResult(result)
-                    self.advanceFrame()
-                    callback(result)  # defer this
-                else:
-                    header_txt = self._header["uid"]
-                    txt = f"Not a valid unit id - {header_txt}, ignoring!!"
-                    _logger.error(txt)
-                    self.resetFrame()
-            else:
+            if not self.checkFrame():
                 break
+            if not self._validate_slave_id(slave, single):
+                header_txt = self._header["uid"]
+                Log.error("Not a valid slave id - {}, ignoring!!", header_txt)
+                self.resetFrame()
+                continue
+
+            frame = self.getFrame()
+            if (result := self.decoder.decode(frame)) is None:
+                raise ModbusIOException("Unable to decode response")
+            self.populateResult(result)
+            self.advanceFrame()
+            callback(result)  # defer this
 
     def buildPacket(self, message):
         """Create a ready to send modbus packet.
@@ -199,19 +138,16 @@ class ModbusAsciiFramer(ModbusFramer):
         :return: The encoded packet
         """
         encoded = message.encode()
-        buffer = struct.pack(ASCII_FRAME_HEADER, message.unit_id, message.function_code)
+        buffer = struct.pack(
+            ASCII_FRAME_HEADER, message.slave_id, message.function_code
+        )
         checksum = computeLRC(encoded + buffer)
 
         packet = bytearray()
-        params = (message.unit_id, message.function_code)
         packet.extend(self._start)
-        packet.extend(
-            ("%02x%02x" % params).encode()  # pylint: disable=consider-using-f-string
-        )
+        packet.extend(f"{message.slave_id:02x}{message.function_code:02x}".encode())
         packet.extend(b2a_hex(encoded))
-        packet.extend(
-            ("%02x" % checksum).encode()  # pylint: disable=consider-using-f-string
-        )
+        packet.extend(f"{checksum:02x}".encode())
         packet.extend(self._end)
         return bytes(packet).upper()
 
