@@ -1,18 +1,13 @@
 """Binary framer."""
 # pylint: disable=missing-type-doc
-import logging
 import struct
 
 from pymodbus.exceptions import ModbusIOException
-from pymodbus.framer import BYTE_ORDER, FRAME_HEADER, ModbusFramer
+from pymodbus.framer.base import BYTE_ORDER, FRAME_HEADER, ModbusFramer
+from pymodbus.logging import Log
 from pymodbus.utilities import checkCRC, computeCRC
 
 
-# --------------------------------------------------------------------------- #
-# Logging
-# --------------------------------------------------------------------------- #
-_logger = logging.getLogger(__name__)
-
 BINARY_FRAME_HEADER = BYTE_ORDER + FRAME_HEADER
 
 # --------------------------------------------------------------------------- #
@@ -51,27 +46,28 @@ class ModbusBinaryFramer(ModbusFramer):
 
         :param decoder: The decoder implementation to use
         """
-        self._buffer = b""
-        self._header = {"crc": 0x0000, "len": 0, "uid": 0x00}
+        super().__init__(decoder, client)
+        # self._header.update({"crc": 0x0000})
         self._hsize = 0x01
         self._start = b"\x7b"  # {
         self._end = b"\x7d"  # }
         self._repeat = [b"}"[0], b"{"[0]]  # python3 hack
-        self.decoder = decoder
-        self.client = client
 
     # ----------------------------------------------------------------------- #
     # Private Helper Functions
     # ----------------------------------------------------------------------- #
+    def _process(self, callback, error=False):
+        """Process incoming packets irrespective error condition."""
+
     def decode_data(self, data):
         """Decode data."""
         if len(data) > self._hsize:
             uid = struct.unpack(">B", data[1:2])[0]
             fcode = struct.unpack(">B", data[2:3])[0]
-            return {"unit": uid, "fcode": fcode}
+            return {"slave": uid, "fcode": fcode}
         return {}
 
-    def checkFrame(self):
+    def checkFrame(self) -> bool:
         """Check and decode the next frame.
 
         :returns: True if we are successful, False otherwise
@@ -90,7 +86,7 @@ class ModbusBinaryFramer(ModbusFramer):
             return checkCRC(data, self._header["crc"])
         return False
 
-    def advanceFrame(self):
+    def advanceFrame(self) -> None:
         """Skip over the current framed message.
 
         This allows us to skip over the current message after we have processed
@@ -100,7 +96,7 @@ class ModbusBinaryFramer(ModbusFramer):
         self._buffer = self._buffer[self._header["len"] + 2 :]
         self._header = {"crc": 0x0000, "len": 0, "uid": 0x00}
 
-    def isFrameReady(self):
+    def isFrameReady(self) -> bool:
         """Check if we should continue decode logic.
 
         This is meant to be used in a while loop in the decoding phase to let
@@ -110,16 +106,6 @@ class ModbusBinaryFramer(ModbusFramer):
         """
         return len(self._buffer) > 1
 
-    def addToFrame(self, message):
-        """Add the next message to the frame buffer.
-
-        This should be used before the decoding while loop to add the received
-        data to the buffer handle.
-
-        :param message: The most recent packet
-        """
-        self._buffer += message
-
     def getFrame(self):
         """Get the next frame from the buffer.
 
@@ -132,63 +118,26 @@ class ModbusBinaryFramer(ModbusFramer):
             return buffer
         return b""
 
-    def populateResult(self, result):
-        """Populate the modbus result header.
-
-        The serial packets do not have any header information
-        that is copied.
-
-        :param result: The response packet
-        """
-        result.unit_id = self._header["uid"]
-
     # ----------------------------------------------------------------------- #
     # Public Member Functions
     # ----------------------------------------------------------------------- #
-    def processIncomingPacket(
-        self, data, callback, unit, **kwargs
-    ):  # pylint: disable=arguments-differ
-        """Process new packet pattern.
-
-        This takes in a new request packet, adds it to the current
-        packet stream, and performs framing on it. That is, checks
-        for complete messages, and once found, will process all that
-        exist.  This handles the case when we read N + 1 or 1 // N
-        messages at a time instead of 1.
-
-        The processed and decoded messages are pushed to the callback
-        function to process and send.
-
-        :param data: The new packet data
-        :param callback: The function to send results to
-        :param unit: Process if unit id matches, ignore otherwise (could be a
-               list of unit ids (server) or single unit id(client/server)
-        :param kwargs:
-        :raises ModbusIOException:
-        """
-        self.addToFrame(data)
-        if not isinstance(unit, (list, tuple)):
-            unit = [unit]
-        single = kwargs.get("single", False)
+    def frameProcessIncomingPacket(self, single, callback, slave, _tid=None, **kwargs):
+        """Process new packet pattern."""
         while self.isFrameReady():
-            if self.checkFrame():
-                if self._validate_unit_id(unit, single):
-                    if (result := self.decoder.decode(self.getFrame())) is None:
-                        raise ModbusIOException("Unable to decode response")
-                    self.populateResult(result)
-                    self.advanceFrame()
-                    callback(result)  # defer or push to a thread?
-                else:
-                    header_txt = self._header["uid"]
-                    txt = f"Not a valid unit id - {header_txt}, ignoring!!"
-                    _logger.debug(txt)
-                    self.resetFrame()
-                    break
-
-            else:
-                _logger.debug("Frame check failed, ignoring!!")
+            if not self.checkFrame():
+                Log.debug("Frame check failed, ignoring!!")
                 self.resetFrame()
                 break
+            if not self._validate_slave_id(slave, single):
+                header_txt = self._header["uid"]
+                Log.debug("Not a valid slave id - {}, ignoring!!", header_txt)
+                self.resetFrame()
+                break
+            if (result := self.decoder.decode(self.getFrame())) is None:
+                raise ModbusIOException("Unable to decode response")
+            self.populateResult(result)
+            self.advanceFrame()
+            callback(result)  # defer or push to a thread?
 
     def buildPacket(self, message):
         """Create a ready to send modbus packet.
@@ -198,7 +147,7 @@ class ModbusBinaryFramer(ModbusFramer):
         """
         data = self._preflight(message.encode())
         packet = (
-            struct.pack(BINARY_FRAME_HEADER, message.unit_id, message.function_code)
+            struct.pack(BINARY_FRAME_HEADER, message.slave_id, message.function_code)
             + data
         )
         packet += struct.pack(">H", computeCRC(packet))
@@ -221,17 +170,5 @@ class ModbusBinaryFramer(ModbusFramer):
             array.append(item)
         return bytes(array)
 
-    def resetFrame(self):  # pylint: disable=invalid-name
-        """Reset the entire message frame.
-
-        This allows us to skip ovver errors that may be in the stream.
-        It is hard to know if we are simply out of sync or if there is
-        an error in the stream as we have no way to check the start or
-        end of the message (python just doesn"t have the resolution to
-        check for millisecond delays).
-        """
-        self._buffer = b""
-        self._header = {"crc": 0x0000, "len": 0, "uid": 0x00}
-
 
 # __END__
