@@ -1,6 +1,5 @@
 """RTU framer."""
 # pylint: disable=missing-type-doc
-import logging
 import struct
 import time
 
@@ -8,20 +7,11 @@ from pymodbus.exceptions import (
     InvalidMessageReceivedException,
     ModbusIOException,
 )
-from pymodbus.framer import BYTE_ORDER, FRAME_HEADER, ModbusFramer
-from pymodbus.utilities import (
-    ModbusTransactionState,
-    checkCRC,
-    computeCRC,
-    hexlify_packets,
-)
+from pymodbus.framer.base import BYTE_ORDER, FRAME_HEADER, ModbusFramer
+from pymodbus.logging import Log
+from pymodbus.utilities import ModbusTransactionState, checkCRC, computeCRC
 
 
-# --------------------------------------------------------------------------- #
-# Logging
-# --------------------------------------------------------------------------- #
-_logger = logging.getLogger(__name__)
-
 RTU_FRAME_HEADER = BYTE_ORDER + FRAME_HEADER
 
 
@@ -68,13 +58,11 @@ class ModbusRtuFramer(ModbusFramer):
 
         :param decoder: The decoder factory implementation to use
         """
-        self._buffer = b""
-        self._header = {"uid": 0x00, "len": 0, "crc": b"\x00\x00"}
+        super().__init__(decoder, client)
         self._hsize = 0x01
         self._end = b"\x0d\x0a"
         self._min_frame_size = 4
-        self.decoder = decoder
-        self.client = client
+        self.function_codes = decoder.lookup.keys() if decoder else {}
 
     # ----------------------------------------------------------------------- #
     # Private Helper Functions
@@ -84,7 +72,7 @@ class ModbusRtuFramer(ModbusFramer):
         if len(data) > self._hsize:
             uid = int(data[0])
             fcode = int(data[1])
-            return {"unit": uid, "fcode": fcode}
+            return {"slave": uid, "fcode": fcode}
         return {}
 
     def checkFrame(self):
@@ -113,25 +101,21 @@ class ModbusRtuFramer(ModbusFramer):
         current frame header handle
         """
         self._buffer = self._buffer[self._header["len"] :]
-        _logger.debug("Frame advanced, resetting header!!")
+        Log.debug("Frame advanced, resetting header!!")
         self._header = {"uid": 0x00, "len": 0, "crc": b"\x00\x00"}
 
-    def resetFrame(self):  # pylint: disable=invalid-name
+    def resetFrame(self):
         """Reset the entire message frame.
 
         This allows us to skip over errors that may be in the stream.
         It is hard to know if we are simply out of sync or if there is
         an error in the stream as we have no way to check the start or
-        end of the message (python just doesn"t have the resolution to
+        end of the message (python just doesn't have the resolution to
         check for millisecond delays).
         """
-        txt = (
-            f"Resetting frame - Current Frame in "
-            f"buffer - {hexlify_packets(self._buffer)}"
-        )
-        _logger.debug(txt)
-        self._buffer = b""
-        self._header = {"uid": 0x00, "len": 0, "crc": b"\x00\x00"}
+        x = self._buffer
+        super().resetFrame()
+        self._buffer = x
 
     def isFrameReady(self):
         """Check if we should continue decode logic.
@@ -153,7 +137,7 @@ class ModbusRtuFramer(ModbusFramer):
 
         return len(self._buffer) >= size if size > 0 else False
 
-    def populateHeader(self, data=None):  # pylint: disable=invalid-name
+    def populateHeader(self, data=None):
         """Try to set the headers `uid`, `len` and `crc`.
 
         This method examines `self._buffer` and writes meta
@@ -164,6 +148,7 @@ class ModbusRtuFramer(ModbusFramer):
         """
         data = data if data is not None else self._buffer
         self._header["uid"] = int(data[0])
+        self._header["tid"] = int(data[0])
         size = self.get_expected_response_length(data)
         self._header["len"] = size
 
@@ -173,13 +158,6 @@ class ModbusRtuFramer(ModbusFramer):
         self._header["crc"] = data[size - 2 : size]
         return size
 
-    def addToFrame(self, message):
-        """Add the received data to the buffer handle.
-
-        :param message: The most recent packet
-        """
-        self._buffer += message
-
     def getFrame(self):
         """Get the next frame from the buffer.
 
@@ -189,8 +167,7 @@ class ModbusRtuFramer(ModbusFramer):
         end = self._header["len"] - 2
         buffer = self._buffer[start:end]
         if end > 0:
-            txt = f"Getting Frame - {hexlify_packets(buffer)}"
-            _logger.debug(txt)
+            Log.debug("Getting Frame - {}", buffer, ":hex")
             return buffer
         return b""
 
@@ -202,55 +179,52 @@ class ModbusRtuFramer(ModbusFramer):
 
         :param result: The response packet
         """
-        result.unit_id = self._header["uid"]
-        result.transaction_id = self._header["uid"]
+        result.slave_id = self._header["uid"]
+        result.transaction_id = self._header["tid"]
+
+    def getFrameStart(self, slaves, broadcast, skip_cur_frame):
+        """Scan buffer for a relevant frame start."""
+        start = 1 if skip_cur_frame else 0
+        if (buf_len := len(self._buffer)) < 4:
+            return False
+        for i in range(start, buf_len - 3):  # <slave id><function code><crc 2 bytes>
+            if not broadcast and self._buffer[i] not in slaves:
+                continue
+            if (
+                self._buffer[i + 1] not in self.function_codes
+                and (self._buffer[i + 1] - 0x80) not in self.function_codes
+            ):
+                continue
+            if i:
+                self._buffer = self._buffer[i:]  # remove preceding trash.
+            return True
+        if buf_len > 3:
+            self._buffer = self._buffer[-3:]
+        return False
 
     # ----------------------------------------------------------------------- #
     # Public Member Functions
     # ----------------------------------------------------------------------- #
-    def processIncomingPacket(
-        self, data, callback, unit, **kwargs
-    ):  # pylint: disable=arguments-differ
-        """Process new packet pattern.
-
-        This takes in a new request packet, adds it to the current
-        packet stream, and performs framing on it. That is, checks
-        for complete messages, and once found, will process all that
-        exist.  This handles the case when we read N + 1 or 1 // N
-        messages at a time instead of 1.
-
-        The processed and decoded messages are pushed to the callback
-        function to process and send.
-
-        :param data: The new packet data
-        :param callback: The function to send results to
-        :param unit: Process if unit id matches, ignore otherwise (could be a
-               list of unit ids (server) or single unit id(client/server)
-        :param kwargs:
-        """
-        if not isinstance(unit, (list, tuple)):
-            unit = [unit]
-        self.addToFrame(data)
-        single = kwargs.get("single", False)
-        while True:
-            if self.isFrameReady():
-                if self.checkFrame():
-                    if self._validate_unit_id(unit, single):
-                        self._process(callback)
-                    else:
-                        header_txt = self._header["uid"]
-                        txt = f"Not a valid unit id - {header_txt}, ignoring!!"
-                        _logger.debug(txt)
-                        self.resetFrame()
-                        break
-                else:
-                    _logger.debug("Frame check failed, ignoring!!")
-                    self.resetFrame()
-                    break
-            else:
-                txt = f"Frame - [{data}] not ready"
-                _logger.debug(txt)
+    def frameProcessIncomingPacket(self, single, callback, slave, _tid=None, **kwargs):
+        """Process new packet pattern."""
+        broadcast = not slave[0]
+        skip_cur_frame = False
+        while self.getFrameStart(slave, broadcast, skip_cur_frame):
+            if not self.isFrameReady():
+                Log.debug("Frame - not ready")
                 break
+            if not self.checkFrame():
+                Log.debug("Frame check failed, ignoring!!")
+                self.resetFrame()
+                skip_cur_frame = True
+                continue
+            if not self._validate_slave_id(slave, single):
+                header_txt = self._header["uid"]
+                Log.debug("Not a valid slave id - {}, ignoring!!", header_txt)
+                self.resetFrame()
+                skip_cur_frame = True
+                continue
+            self._process(callback)
 
     def buildPacket(self, message):
         """Create a ready to send modbus packet.
@@ -259,11 +233,12 @@ class ModbusRtuFramer(ModbusFramer):
         """
         data = message.encode()
         packet = (
-            struct.pack(RTU_FRAME_HEADER, message.unit_id, message.function_code) + data
+            struct.pack(RTU_FRAME_HEADER, message.slave_id, message.function_code)
+            + data
         )
         packet += struct.pack(">H", computeCRC(packet))
-        # Ensure that transaction is actually the unit id for serial comms
-        message.transaction_id = message.unit_id
+        # Ensure that transaction is actually the slave id for serial comms
+        message.transaction_id = message.slave_id
         return packet
 
     def sendPacket(self, message):
@@ -272,25 +247,24 @@ class ModbusRtuFramer(ModbusFramer):
         :param message: Message to be sent over the bus
         :return:
         """
+        super().resetFrame()
         start = time.time()
-        timeout = start + self.client.params.timeout
+        timeout = start + self.client.comm_params.timeout_connect
         while self.client.state != ModbusTransactionState.IDLE:
             if self.client.state == ModbusTransactionState.TRANSACTION_COMPLETE:
                 timestamp = round(time.time(), 6)
-                txt = (
-                    f"Changing state to IDLE - Last Frame End - {self.client.last_frame_end}, "
-                    f"Current Time stamp - {timestamp}"
+                Log.debug(
+                    "Changing state to IDLE - Last Frame End - {} Current Time stamp - {}",
+                    self.client.last_frame_end,
+                    timestamp,
                 )
-                _logger.debug(txt)
-
                 if self.client.last_frame_end:
                     idle_time = self.client.idle_time()
                     if round(timestamp - idle_time, 6) <= self.client.silent_interval:
-                        txt = (
-                            f"Waiting for 3.5 char before next "
-                            f"send - {self.client.silent_interval * 1000} ms"
+                        Log.debug(
+                            "Waiting for 3.5 char before next send - {} ms",
+                            self.client.silent_interval * 1000,
                         )
-                        _logger.debug(txt)
                         time.sleep(self.client.silent_interval)
                 else:
                     # Recovering from last error ??
@@ -299,16 +273,16 @@ class ModbusRtuFramer(ModbusFramer):
             elif self.client.state == ModbusTransactionState.RETRYING:
                 # Simple lets settle down!!!
                 # To check for higher baudrates
-                time.sleep(self.client.params.timeout)
+                time.sleep(self.client.comm_params.timeout_connect)
                 break
             elif time.time() > timeout:
-                _logger.debug(
+                Log.debug(
                     "Spent more time than the read time out, "
                     "resetting the transaction to IDLE"
                 )
                 self.client.state = ModbusTransactionState.IDLE
             else:
-                _logger.debug("Sleeping")
+                Log.debug("Sleeping")
                 time.sleep(self.client.silent_interval)
         size = self.client.send(message)
         self.client.last_frame_end = round(time.time(), 6)
@@ -326,7 +300,7 @@ class ModbusRtuFramer(ModbusFramer):
 
     def _process(self, callback, error=False):
         """Process incoming packets irrespective error condition."""
-        data = self.getRawFrame() if error else self.getFrame()
+        data = self._buffer if error else self.getFrame()
         if (result := self.decoder.decode(data)) is None:
             raise ModbusIOException("Unable to decode request")
         if error and result.function_code < 0x80:
@@ -335,12 +309,6 @@ class ModbusRtuFramer(ModbusFramer):
         self.advanceFrame()
         callback(result)  # defer or push to a thread?
 
-    def getRawFrame(self):  # pylint: disable=invalid-name
-        """Return the complete buffer."""
-        txt = f"Getting Raw Frame - {hexlify_packets(self._buffer)}"
-        _logger.debug(txt)
-        return self._buffer
-
     def get_expected_response_length(self, data):
         """Get the expected response length.
 
