@@ -1,21 +1,15 @@
 """Socket framer."""
 # pylint: disable=missing-type-doc
-import logging
 import struct
 
 from pymodbus.exceptions import (
     InvalidMessageReceivedException,
     ModbusIOException,
 )
-from pymodbus.framer import SOCKET_FRAME_HEADER, ModbusFramer
-from pymodbus.utilities import hexlify_packets
+from pymodbus.framer.base import SOCKET_FRAME_HEADER, ModbusFramer
+from pymodbus.logging import Log
 
 
-# --------------------------------------------------------------------------- #
-# Logging
-# --------------------------------------------------------------------------- #
-_logger = logging.getLogger(__name__)
-
 # --------------------------------------------------------------------------- #
 # Modbus TCP Message
 # --------------------------------------------------------------------------- #
@@ -47,11 +41,8 @@ class ModbusSocketFramer(ModbusFramer):
 
         :param decoder: The decoder factory implementation to use
         """
-        self._buffer = b""
-        self._header = {"tid": 0, "pid": 0, "len": 0, "uid": 0}
+        super().__init__(decoder, client)
         self._hsize = 0x07
-        self.decoder = decoder
-        self.client = client
 
     # ----------------------------------------------------------------------- #
     # Private Helper Functions
@@ -61,21 +52,22 @@ class ModbusSocketFramer(ModbusFramer):
 
         Return true if we were successful.
         """
-        if self.isFrameReady():
-            (
-                self._header["tid"],
-                self._header["pid"],
-                self._header["len"],
-                self._header["uid"],
-            ) = struct.unpack(">HHHB", self._buffer[0 : self._hsize])
+        if not self.isFrameReady():
+            return False
+        (
+            self._header["tid"],
+            self._header["pid"],
+            self._header["len"],
+            self._header["uid"],
+        ) = struct.unpack(">HHHB", self._buffer[0 : self._hsize])
 
-            # someone sent us an error? ignore it
-            if self._header["len"] < 2:
-                self.advanceFrame()
-            # we have at least a complete message, continue
-            elif len(self._buffer) - self._hsize + 1 >= self._header["len"]:
-                return True
-        # we don"t have enough of a message yet, wait
+        # someone sent us an error? ignore it
+        if self._header["len"] < 2:
+            self.advanceFrame()
+        # we have at least a complete message, continue
+        elif len(self._buffer) - self._hsize + 1 >= self._header["len"]:
+            return True
+        # we don't have enough of a message yet, wait
         return False
 
     def advanceFrame(self):
@@ -85,7 +77,7 @@ class ModbusSocketFramer(ModbusFramer):
         it or determined that it contains an error. It also has to reset the
         current frame header handle
         """
-        length = self._hsize + self._header["len"] - 1
+        length = self._hsize + self._header["len"] -1
         self._buffer = self._buffer[length:]
         self._header = {"tid": 0, "pid": 0, "len": 0, "uid": 0}
 
@@ -99,33 +91,14 @@ class ModbusSocketFramer(ModbusFramer):
         """
         return len(self._buffer) > self._hsize
 
-    def addToFrame(self, message):
-        """Add new packet data to the current frame buffer.
-
-        :param message: The most recent packet
-        """
-        self._buffer += message
-
     def getFrame(self):
         """Return the next frame from the buffered data.
 
         :returns: The next full frame buffer
         """
-        length = self._hsize + self._header["len"] - 1
+        length = self._hsize + self._header["len"]
         return self._buffer[self._hsize : length]
 
-    def populateResult(self, result):
-        """Populate the modbus result.
-
-        With the transport specific header
-        information (pid, tid, uid, checksum, etc)
-
-        :param result: The response packet
-        """
-        result.transaction_id = self._header["tid"]
-        result.protocol_id = self._header["pid"]
-        result.unit_id = self._header["uid"]
-
     # ----------------------------------------------------------------------- #
     # Public Member Functions
     # ----------------------------------------------------------------------- #
@@ -139,14 +112,12 @@ class ModbusSocketFramer(ModbusFramer):
                 "tid": tid,
                 "pid": pid,
                 "length": length,
-                "unit": uid,
+                "slave": uid,
                 "fcode": fcode,
             }
         return {}
 
-    def processIncomingPacket(  # pylint: disable=arguments-differ
-        self, data, callback, unit, **kwargs
-    ):
+    def frameProcessIncomingPacket(self, single, callback, slave, tid=None, **kwargs):
         """Process new packet pattern.
 
         This takes in a new request packet, adds it to the current
@@ -157,65 +128,32 @@ class ModbusSocketFramer(ModbusFramer):
 
         The processed and decoded messages are pushed to the callback
         function to process and send.
-
-        :param data: The new packet data
-        :param callback: The function to send results to
-        :param unit: Process if unit id matches, ignore otherwise (could be a
-               list of unit ids (server) or single unit id(client/server)
-        :param kwargs:
         """
-        if not isinstance(unit, (list, tuple)):
-            unit = [unit]
-        single = kwargs.get("single", False)
-        txt = f"Processing: {hexlify_packets(data)}"
-        _logger.debug(txt)
-        self.addToFrame(data)
         while True:
-            if self.isFrameReady():
-                if self.checkFrame():
-                    if self._validate_unit_id(unit, single):
-                        self._process(callback)
-                    else:
-                        header_txt = self._header["uid"]
-                        txt = f"Not a valid unit id - {header_txt}, ignoring!!"
-                        _logger.debug(txt)
-                        self.resetFrame()
-                else:
-                    _logger.debug("Frame check failed, ignoring!!")
-                    self.resetFrame()
-            else:
-                if len(self._buffer):
-                    # Possible error ???
-                    if self._header["len"] < 2:
-                        self._process(callback, error=True)
-                break
+            if not self.checkFrame():
+                Log.debug("Frame check failed, ignoring!!")
+                return
+            if not self._validate_slave_id(slave, single):
+                header_txt = self._header["uid"]
+                Log.debug("Not a valid slave id - {}, ignoring!!", header_txt)
+                self.resetFrame()
+                return
+            self._process(callback, tid)
 
-    def _process(self, callback, error=False):
+    def _process(self, callback, tid, error=False):
         """Process incoming packets irrespective error condition."""
-        data = self.getRawFrame() if error else self.getFrame()
+        data = self._buffer if error else self.getFrame()
         if (result := self.decoder.decode(data)) is None:
+            self.resetFrame()
             raise ModbusIOException("Unable to decode request")
         if error and result.function_code < 0x80:
             raise InvalidMessageReceivedException(result)
         self.populateResult(result)
         self.advanceFrame()
-        callback(result)  # defer or push to a thread?
-
-    def resetFrame(self):  # pylint: disable=invalid-name
-        """Reset the entire message frame.
-
-        This allows us to skip ovver errors that may be in the stream.
-        It is hard to know if we are simply out of sync or if there is
-        an error in the stream as we have no way to check the start or
-        end of the message (python just doesn"t have the resolution to
-        check for millisecond delays).
-        """
-        self._buffer = b""
-        self._header = {"tid": 0, "pid": 0, "len": 0, "uid": 0}
-
-    def getRawFrame(self):  # pylint: disable=invalid-name
-        """Return the complete buffer."""
-        return self._buffer
+        if tid and tid != result.transaction_id:
+            self.resetFrame()
+        else:
+            callback(result)  # defer or push to a thread?
 
     def buildPacket(self, message):
         """Create a ready to send modbus packet.
@@ -228,7 +166,7 @@ class ModbusSocketFramer(ModbusFramer):
             message.transaction_id,
             message.protocol_id,
             len(data) + 2,
-            message.unit_id,
+            message.slave_id,
             message.function_code,
         )
         packet += data
