@@ -1,28 +1,22 @@
 """TLS framer."""
 # pylint: disable=missing-type-doc
-import logging
 import struct
 
 from pymodbus.exceptions import (
     InvalidMessageReceivedException,
     ModbusIOException,
 )
-from pymodbus.framer import TLS_FRAME_HEADER, ModbusFramer
-from pymodbus.utilities import hexlify_packets
+from pymodbus.framer.base import TLS_FRAME_HEADER, ModbusFramer
+from pymodbus.logging import Log
 
 
-# --------------------------------------------------------------------------- #
-# Logging
-# --------------------------------------------------------------------------- #
-_logger = logging.getLogger(__name__)
-
 # --------------------------------------------------------------------------- #
 # Modbus TLS Message
 # --------------------------------------------------------------------------- #
 
 
 class ModbusTlsFramer(ModbusFramer):
-    """Modbus TLS Frame controller
+    """Modbus TLS Frame controller.
 
     No prefix MBAP header before decrypted PDU is used as a message frame for
     Modbus Security Application Protocol.  It allows us to easily separate
@@ -39,11 +33,8 @@ class ModbusTlsFramer(ModbusFramer):
 
         :param decoder: The decoder factory implementation to use
         """
-        self._buffer = b""
-        self._header = {}
+        super().__init__(decoder, client)
         self._hsize = 0x0
-        self.decoder = decoder
-        self.client = client
 
     # ----------------------------------------------------------------------- #
     # Private Helper Functions
@@ -57,7 +48,7 @@ class ModbusTlsFramer(ModbusFramer):
             # we have at least a complete message, continue
             if len(self._buffer) - self._hsize >= 1:
                 return True
-        # we don"t have enough of a message yet, wait
+        # we don't have enough of a message yet, wait
         return False
 
     def advanceFrame(self):
@@ -80,13 +71,6 @@ class ModbusTlsFramer(ModbusFramer):
         """
         return len(self._buffer) > self._hsize
 
-    def addToFrame(self, message):
-        """Add new packet data to the current frame buffer.
-
-        :param message: The most recent packet
-        """
-        self._buffer += message
-
     def getFrame(self):
         """Return the next frame from the buffered data.
 
@@ -94,16 +78,6 @@ class ModbusTlsFramer(ModbusFramer):
         """
         return self._buffer[self._hsize :]
 
-    def populateResult(self, result):
-        """Populate the modbus result.
-
-        With the transport specific header
-        information (no header before PDU in decrypted message)
-
-        :param result: The response packet
-        """
-        return
-
     # ----------------------------------------------------------------------- #
     # Public Member Functions
     # ----------------------------------------------------------------------- #
@@ -114,49 +88,23 @@ class ModbusTlsFramer(ModbusFramer):
             return {"fcode": fcode}
         return {}
 
-    def processIncomingPacket(
-        self, data, callback, unit, **kwargs
-    ):  # pylint: disable=arguments-differ
-        """Process new packet pattern.
-
-        This takes in a new request packet, adds it to the current
-        packet stream, and performs framing on it. That is, checks
-        for complete messages, and once found, will process all that
-        exist.  This handles the case when we read N + 1 or 1 // N
-        messages at a time instead of 1.
-
-        The processed and decoded messages are pushed to the callback
-        function to process and send.
-
-        :param data: The new packet data
-        :param callback: The function to send results to
-        :param unit: Process if unit id matches, ignore otherwise (could be a
-               list of unit ids (server) or single unit id(client/server)
-        :param kwargs:
-        """
-        if not isinstance(unit, (list, tuple)):
-            unit = [unit]
-        # no unit id for Modbus Security Application Protocol
-        single = kwargs.get("single", True)
-        txt = f"Processing: {hexlify_packets(data)}"
-        _logger.debug(txt)
-        self.addToFrame(data)
-
-        if self.isFrameReady():
-            if self.checkFrame():
-                if self._validate_unit_id(unit, single):
-                    self._process(callback)
-                else:
-                    txt = f"Not in valid unit id - {unit}, ignoring!!"
-                    _logger.debug(txt)
-                    self.resetFrame()
-            else:
-                _logger.debug("Frame check failed, ignoring!!")
-                self.resetFrame()
+    def frameProcessIncomingPacket(self, single, callback, slave, _tid=None, **kwargs):
+        """Process new packet pattern."""
+        # no slave id for Modbus Security Application Protocol
+        if not self.isFrameReady():
+            return
+        if not self.checkFrame():
+            Log.debug("Frame check failed, ignoring!!")
+            self.resetFrame()
+            return
+        if not self._validate_slave_id(slave, single):
+            Log.debug("Not in valid slave id - {}, ignoring!!", slave)
+            self.resetFrame()
+        self._process(callback)
 
     def _process(self, callback, error=False):
         """Process incoming packets irrespective error condition."""
-        data = self.getRawFrame() if error else self.getFrame()
+        data = self._buffer if error else self.getFrame()
         if (result := self.decoder.decode(data)) is None:
             raise ModbusIOException("Unable to decode request")
         if error and result.function_code < 0x80:
@@ -165,21 +113,6 @@ class ModbusTlsFramer(ModbusFramer):
         self.advanceFrame()
         callback(result)  # defer or push to a thread?
 
-    def resetFrame(self):  # pylint: disable=invalid-name
-        """Reset the entire message frame.
-
-        This allows us to skip ovver errors that may be in the stream.
-        It is hard to know if we are simply out of sync or if there is
-        an error in the stream as we have no way to check the start or
-        end of the message (python just doesn"t have the resolution to
-        check for millisecond delays).
-        """
-        self._buffer = b""
-
-    def getRawFrame(self):  # pylint: disable=invalid-name
-        """Return the complete buffer."""
-        return self._buffer
-
     def buildPacket(self, message):
         """Create a ready to send modbus packet.
 
