@@ -2,10 +2,22 @@
 
 Currently not all implemented
 """
+
+__all__ = [
+    "ReadExceptionStatusRequest",
+    "ReadExceptionStatusResponse",
+    "GetCommEventCounterRequest",
+    "GetCommEventCounterResponse",
+    "GetCommEventLogRequest",
+    "GetCommEventLogResponse",
+    "ReportSlaveIdRequest",
+    "ReportSlaveIdResponse",
+]
+
 # pylint: disable=missing-type-doc
 import struct
 
-from pymodbus.constants import Defaults, ModbusStatus
+from pymodbus.constants import ModbusStatus
 from pymodbus.device import DeviceInformationFactory, ModbusControlBlock
 from pymodbus.pdu import ModbusRequest, ModbusResponse
 
@@ -25,11 +37,12 @@ class ReadExceptionStatusRequest(ModbusRequest):
     """
 
     function_code = 0x07
+    function_code_name = "read_exception_status"
     _rtu_frame_size = 4
 
-    def __init__(self, unit=None, **kwargs):
+    def __init__(self, slave=None, **kwargs):
         """Initialize a new instance."""
-        ModbusRequest.__init__(self, unit=unit, **kwargs)
+        ModbusRequest.__init__(self, slave=slave, **kwargs)
 
     def encode(self):
         """Encode the message."""
@@ -41,7 +54,7 @@ class ReadExceptionStatusRequest(ModbusRequest):
         :param data: The incoming data
         """
 
-    def execute(self, context=None):  # pylint: disable=unused-argument
+    def execute(self, _context=None):
         """Run a read exception status request against the store.
 
         :returns: The populated response
@@ -75,7 +88,7 @@ class ReadExceptionStatusResponse(ModbusResponse):
         :param status: The status response to report
         """
         ModbusResponse.__init__(self, **kwargs)
-        self.status = status
+        self.status = status if status < 256 else 255
 
     def encode(self):
         """Encode the response.
@@ -113,13 +126,13 @@ class ReadExceptionStatusResponse(ModbusResponse):
 class GetCommEventCounterRequest(ModbusRequest):
     """This function code is used to get a status word.
 
-    And an event count from the remote device"s communication event counter.
+    And an event count from the remote device's communication event counter.
 
     By fetching the current count before and after a series of messages, a
     client can determine whether the messages were handled normally by the
     remote device.
 
-    The device"s event counter is incremented once  for each successful
+    The device's event counter is incremented once  for each successful
     message completion. It is not incremented for exception responses,
     poll commands, or fetch event counter commands.
 
@@ -129,6 +142,7 @@ class GetCommEventCounterRequest(ModbusRequest):
     """
 
     function_code = 0x0B
+    function_code_name = "get_event_counter"
     _rtu_frame_size = 4
 
     def __init__(self, **kwargs):
@@ -145,7 +159,7 @@ class GetCommEventCounterRequest(ModbusRequest):
         :param data: The incoming data
         """
 
-    def execute(self, context=None):  # pylint: disable=unused-argument
+    def execute(self, _context=None):
         """Run a read exception status request against the store.
 
         :returns: The populated response
@@ -189,9 +203,9 @@ class GetCommEventCounterResponse(ModbusResponse):
         :returns: The byte encoded message
         """
         if self.status:
-            ready = ModbusStatus.Ready
+            ready = ModbusStatus.READY
         else:
-            ready = ModbusStatus.Waiting
+            ready = ModbusStatus.WAITING
         return struct.pack(">HH", ready, self.count)
 
     def decode(self, data):
@@ -200,7 +214,7 @@ class GetCommEventCounterResponse(ModbusResponse):
         :param data: The packet data to decode
         """
         ready, self.count = struct.unpack(">HH", data)
-        self.status = ready == ModbusStatus.Ready
+        self.status = ready == ModbusStatus.READY
 
     def __str__(self):
         """Build a representation of the response.
@@ -239,6 +253,7 @@ class GetCommEventLogRequest(ModbusRequest):
     """
 
     function_code = 0x0C
+    function_code_name = "get_event_log"
     _rtu_frame_size = 4
 
     def __init__(self, **kwargs):
@@ -255,7 +270,7 @@ class GetCommEventLogRequest(ModbusRequest):
         :param data: The incoming data
         """
 
-    def execute(self, context=None):  # pylint: disable=unused-argument
+    def execute(self, _context=None):
         """Run a read exception status request against the store.
 
         :returns: The populated response
@@ -308,9 +323,9 @@ class GetCommEventLogResponse(ModbusResponse):
         :returns: The byte encoded message
         """
         if self.status:
-            ready = ModbusStatus.Ready
+            ready = ModbusStatus.READY
         else:
-            ready = ModbusStatus.Waiting
+            ready = ModbusStatus.WAITING
         packet = struct.pack(">B", 6 + len(self.events))
         packet += struct.pack(">H", ready)
         packet += struct.pack(">HH", self.event_count, self.message_count)
@@ -324,7 +339,7 @@ class GetCommEventLogResponse(ModbusResponse):
         """
         length = int(data[0])
         status = struct.unpack(">H", data[1:3])[0]
-        self.status = status == ModbusStatus.Ready
+        self.status = status == ModbusStatus.READY
         self.event_count = struct.unpack(">H", data[3:5])[0]
         self.message_count = struct.unpack(">H", data[5:7])[0]
 
@@ -359,15 +374,16 @@ class ReportSlaveIdRequest(ModbusRequest):
     """
 
     function_code = 0x11
+    function_code_name = "report_slave_id"
     _rtu_frame_size = 4
 
-    def __init__(self, unit=Defaults.Slave, **kwargs):
+    def __init__(self, slave=0, **kwargs):
         """Initialize a new instance.
 
-        :param unit: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
 
         """
-        ModbusRequest.__init__(self, unit, **kwargs)
+        ModbusRequest.__init__(self, slave, **kwargs)
 
     def encode(self):
         """Encode the message."""
@@ -437,9 +453,9 @@ class ReportSlaveIdResponse(ModbusResponse):
         :returns: The byte encoded message
         """
         if self.status:
-            status = ModbusStatus.SlaveOn
+            status = ModbusStatus.SLAVE_ON
         else:
-            status = ModbusStatus.SlaveOff
+            status = ModbusStatus.SLAVE_OFF
         length = len(self.identifier) + 1
         packet = struct.pack(">B", length)
         packet += self.identifier  # we assume it is already encoded
@@ -457,36 +473,11 @@ class ReportSlaveIdResponse(ModbusResponse):
         self.byte_count = int(data[0])
         self.identifier = data[1 : self.byte_count + 1]
         status = int(data[-1])
-        self.status = status == ModbusStatus.SlaveOn
+        self.status = status == ModbusStatus.SLAVE_ON
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Build a representation of the response.
 
         :returns: The string representation of the response
         """
-        arguments = (self.function_code, self.identifier, self.status)
-        return (
-            "ReportSlaveIdResponse(%s, %s, %s)"  # pylint: disable=consider-using-f-string
-            % arguments
-        )
-
-
-# ---------------------------------------------------------------------------#
-#  TODO Make these only work on serial # pylint: disable=fixme
-# ---------------------------------------------------------------------------#
-# report device identification 43, 14
-
-
-# ---------------------------------------------------------------------------#
-#  Exported symbols
-# ---------------------------------------------------------------------------#
-__all__ = [
-    "ReadExceptionStatusRequest",
-    "ReadExceptionStatusResponse",
-    "GetCommEventCounterRequest",
-    "GetCommEventCounterResponse",
-    "GetCommEventLogRequest",
-    "GetCommEventLogResponse",
-    "ReportSlaveIdRequest",
-    "ReportSlaveIdResponse",
-]
+        return f"ReportSlaveIdResponse({self.function_code}, {self.identifier}, {self.status})"
