@@ -3,30 +3,30 @@
 A collection of utilities for building and decoding
 modbus messages payloads.
 """
+from __future__ import annotations
+
+
+__all__ = [
+    "BinaryPayloadBuilder",
+    "BinaryPayloadDecoder",
+]
+
 # pylint: disable=missing-type-doc
-import logging
 from struct import pack, unpack
 
 from pymodbus.constants import Endian
 from pymodbus.exceptions import ParameterException
-from pymodbus.interfaces import IPayloadBuilder
+from pymodbus.logging import Log
 from pymodbus.utilities import (
-    make_byte_string,
     pack_bitstring,
     unpack_bitstring,
 )
 
 
-# --------------------------------------------------------------------------- #
-# Logging
-# --------------------------------------------------------------------------- #
-_logger = logging.getLogger(__name__)
-
-
 WC = {"b": 1, "h": 2, "e": 2, "i": 4, "l": 4, "q": 8, "f": 4, "d": 8}
 
 
-class BinaryPayloadBuilder(IPayloadBuilder):
+class BinaryPayloadBuilder:
     """A utility that helps build payload messages to be written with the various modbus messages.
 
     It really is just a simple wrapper around the struct module,
@@ -40,7 +40,7 @@ class BinaryPayloadBuilder(IPayloadBuilder):
     """
 
     def __init__(
-        self, payload=None, byteorder=Endian.Little, wordorder=Endian.Big, repack=False
+        self, payload=None, byteorder=Endian.LITTLE, wordorder=Endian.BIG, repack=False
     ):
         """Initialize a new instance of the payload builder.
 
@@ -68,14 +68,12 @@ class BinaryPayloadBuilder(IPayloadBuilder):
         :param value: Value to be packed
         :return:
         """
-        value = pack(
-            "!{}".format(fstring), value  # pylint: disable=consider-using-f-string
-        )
+        value = pack(f"!{fstring}", value)
         wordorder = WC.get(fstring.lower()) // 2
         upperbyte = f"!{wordorder}H"
         payload = unpack(upperbyte, value)
 
-        if self._wordorder == Endian.Little:
+        if self._wordorder == Endian.LITTLE:
             payload = list(reversed(payload))
 
         fstring = self._byteorder + "H"
@@ -84,21 +82,18 @@ class BinaryPayloadBuilder(IPayloadBuilder):
 
         return payload
 
-    def to_string(self):
-        """Return the payload buffer as a string.
-
-        :returns: The payload buffer as a string
-        """
+    def encode(self) -> bytes:
+        """Get the payload buffer encoded in bytes."""
         return b"".join(self._payload)
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Return the payload buffer as a string.
 
         :returns: The payload buffer as a string
         """
-        return self.to_string().decode("utf-8")
+        return self.encode().decode("utf-8")
 
-    def reset(self):
+    def reset(self) -> None:
         """Reset the payload buffer."""
         self._payload = []
 
@@ -114,10 +109,10 @@ class BinaryPayloadBuilder(IPayloadBuilder):
             payload = [unpack(self._byteorder + "H", value)[0] for value in payload]
         else:
             payload = [unpack(fstring, value)[0] for value in payload]
-        _logger.debug(payload)
+        Log.debug("{}", payload)
         return payload
 
-    def to_coils(self):
+    def to_coils(self) -> list[bool]:
         """Convert the payload buffer into a coil layout that can be used as a context block.
 
         :returns: The coil layout to use as a block
@@ -126,7 +121,7 @@ class BinaryPayloadBuilder(IPayloadBuilder):
         coils = [bool(int(bit)) for reg in payload for bit in format(reg, "016b")]
         return coils
 
-    def build(self):
+    def build(self) -> list[bytes]:
         """Return the payload buffer as a list.
 
         This list is two bytes per element and can
@@ -134,12 +129,12 @@ class BinaryPayloadBuilder(IPayloadBuilder):
 
         :returns: The payload buffer as a list
         """
-        string = self.to_string()
-        length = len(string)
-        string = string + (b"\x00" * (length % 2))
-        return [string[i : i + 2] for i in range(0, length, 2)]
+        buffer = self.encode()
+        length = len(buffer)
+        buffer += b"\x00" * (length % 2)
+        return [buffer[i : i + 2] for i in range(0, length, 2)]
 
-    def add_bits(self, values):
+    def add_bits(self, values: list[bool]) -> None:
         """Add a collection of bits to be encoded.
 
         If these are less than a multiple of eight,
@@ -151,7 +146,7 @@ class BinaryPayloadBuilder(IPayloadBuilder):
         value = pack_bitstring(values)
         self._payload.append(value)
 
-    def add_8bit_uint(self, value):
+    def add_8bit_uint(self, value: int) -> None:
         """Add a 8 bit unsigned int to the buffer.
 
         :param value: The value to add to the buffer
@@ -159,7 +154,7 @@ class BinaryPayloadBuilder(IPayloadBuilder):
         fstring = self._byteorder + "B"
         self._payload.append(pack(fstring, value))
 
-    def add_16bit_uint(self, value):
+    def add_16bit_uint(self, value: int) -> None:
         """Add a 16 bit unsigned int to the buffer.
 
         :param value: The value to add to the buffer
@@ -167,7 +162,7 @@ class BinaryPayloadBuilder(IPayloadBuilder):
         fstring = self._byteorder + "H"
         self._payload.append(pack(fstring, value))
 
-    def add_32bit_uint(self, value):
+    def add_32bit_uint(self, value: int) -> None:
         """Add a 32 bit unsigned int to the buffer.
 
         :param value: The value to add to the buffer
@@ -177,7 +172,7 @@ class BinaryPayloadBuilder(IPayloadBuilder):
         p_string = self._pack_words(fstring, value)
         self._payload.append(p_string)
 
-    def add_64bit_uint(self, value):
+    def add_64bit_uint(self, value: int) -> None:
         """Add a 64 bit unsigned int to the buffer.
 
         :param value: The value to add to the buffer
@@ -186,7 +181,7 @@ class BinaryPayloadBuilder(IPayloadBuilder):
         p_string = self._pack_words(fstring, value)
         self._payload.append(p_string)
 
-    def add_8bit_int(self, value):
+    def add_8bit_int(self, value: int) -> None:
         """Add a 8 bit signed int to the buffer.
 
         :param value: The value to add to the buffer
@@ -194,7 +189,7 @@ class BinaryPayloadBuilder(IPayloadBuilder):
         fstring = self._byteorder + "b"
         self._payload.append(pack(fstring, value))
 
-    def add_16bit_int(self, value):
+    def add_16bit_int(self, value: int) -> None:
         """Add a 16 bit signed int to the buffer.
 
         :param value: The value to add to the buffer
@@ -202,7 +197,7 @@ class BinaryPayloadBuilder(IPayloadBuilder):
         fstring = self._byteorder + "h"
         self._payload.append(pack(fstring, value))
 
-    def add_32bit_int(self, value):
+    def add_32bit_int(self, value: int) -> None:
         """Add a 32 bit signed int to the buffer.
 
         :param value: The value to add to the buffer
@@ -211,7 +206,7 @@ class BinaryPayloadBuilder(IPayloadBuilder):
         p_string = self._pack_words(fstring, value)
         self._payload.append(p_string)
 
-    def add_64bit_int(self, value):
+    def add_64bit_int(self, value: int) -> None:
         """Add a 64 bit signed int to the buffer.
 
         :param value: The value to add to the buffer
@@ -220,7 +215,7 @@ class BinaryPayloadBuilder(IPayloadBuilder):
         p_string = self._pack_words(fstring, value)
         self._payload.append(p_string)
 
-    def add_16bit_float(self, value):
+    def add_16bit_float(self, value: float) -> None:
         """Add a 16 bit float to the buffer.
 
         :param value: The value to add to the buffer
@@ -229,7 +224,7 @@ class BinaryPayloadBuilder(IPayloadBuilder):
         p_string = self._pack_words(fstring, value)
         self._payload.append(p_string)
 
-    def add_32bit_float(self, value):
+    def add_32bit_float(self, value: float) -> None:
         """Add a 32 bit float to the buffer.
 
         :param value: The value to add to the buffer
@@ -238,7 +233,7 @@ class BinaryPayloadBuilder(IPayloadBuilder):
         p_string = self._pack_words(fstring, value)
         self._payload.append(p_string)
 
-    def add_64bit_float(self, value):
+    def add_64bit_float(self, value: float) -> None:
         """Add a 64 bit float(double) to the buffer.
 
         :param value: The value to add to the buffer
@@ -247,14 +242,13 @@ class BinaryPayloadBuilder(IPayloadBuilder):
         p_string = self._pack_words(fstring, value)
         self._payload.append(p_string)
 
-    def add_string(self, value):
+    def add_string(self, value: str) -> None:
         """Add a string to the buffer.
 
         :param value: The value to add to the buffer
         """
-        value = make_byte_string(value)
         fstring = self._byteorder + str(len(value)) + "s"
-        self._payload.append(pack(fstring, value))
+        self._payload.append(pack(fstring, value.encode()))
 
 
 class BinaryPayloadDecoder:
@@ -269,7 +263,7 @@ class BinaryPayloadDecoder:
         second  = decoder.decode_16bit_uint()
     """
 
-    def __init__(self, payload, byteorder=Endian.Little, wordorder=Endian.Big):
+    def __init__(self, payload, byteorder=Endian.LITTLE, wordorder=Endian.BIG):
         """Initialize a new payload decoder.
 
         :param payload: The payload to decode with
@@ -282,11 +276,11 @@ class BinaryPayloadDecoder:
         self._wordorder = wordorder
 
     @classmethod
-    def fromRegisters(  # pylint: disable=invalid-name
+    def fromRegisters(
         cls,
         registers,
-        byteorder=Endian.Little,
-        wordorder=Endian.Big,
+        byteorder=Endian.LITTLE,
+        wordorder=Endian.BIG,
     ):
         """Initialize a payload decoder.
 
@@ -302,7 +296,7 @@ class BinaryPayloadDecoder:
         :returns: An initialized PayloadDecoder
         :raises ParameterException:
         """
-        _logger.debug(registers)
+        Log.debug("{}", registers)
         if isinstance(registers, list):  # repack into flat binary
             payload = b"".join(pack("!H", x) for x in registers)
             return cls(payload, byteorder, wordorder)
@@ -315,25 +309,16 @@ class BinaryPayloadDecoder:
         return chunks
 
     @classmethod
-    def fromCoils(  # pylint: disable=invalid-name
+    def fromCoils(
         cls,
         coils,
-        byteorder=Endian.Little,
-        wordorder=Endian.Big,
-    ):  # pylint: disable=unused-argument
-        """Initialize a payload decoder with the result of reading of coils.
-
-        The coils are treated as a list of bit(boolean) values.
-
-        :param coils: The coil results to initialize with
-        :param byteorder: The endianness of the payload
-        :param wordorder: The endianness of the payload
-        :returns: An initialized PayloadDecoder
-        :raises ParameterException:
-        """
+        byteorder=Endian.LITTLE,
+        _wordorder=Endian.BIG,
+    ):
+        """Initialize a payload decoder with the result of reading of coils."""
         if isinstance(coils, list):
             payload = b""
-            if padding := len(coils) % 8:  # Pad zero"s
+            if padding := len(coils) % 8:  # Pad zeros
                 extra = [False] * padding
                 coils = extra + coils
             chunks = cls.bit_chunks(coils)
@@ -354,15 +339,14 @@ class BinaryPayloadDecoder:
         :param handle: Value to be unpacked
         :return:
         """
-        handle = make_byte_string(handle)
         wc_value = WC.get(fstring.lower()) // 2
         handle = unpack(f"!{wc_value}H", handle)
-        if self._wordorder == Endian.Little:
+        if self._wordorder == Endian.LITTLE:
             handle = list(reversed(handle))
 
         # Repack as unsigned Integer
         handle = [pack(self._byteorder + "H", p) for p in handle]
-        _logger.debug(handle)
+        Log.debug("handle: {}", handle)
         handle = b"".join(handle)
         return handle
 
@@ -375,7 +359,6 @@ class BinaryPayloadDecoder:
         self._pointer += 1
         fstring = self._byteorder + "B"
         handle = self._payload[self._pointer - 1 : self._pointer]
-        handle = make_byte_string(handle)
         return unpack(fstring, handle)[0]
 
     def decode_bits(self, package_len=1):
@@ -383,7 +366,6 @@ class BinaryPayloadDecoder:
         self._pointer += package_len
         # fstring = self._endian + "B"
         handle = self._payload[self._pointer - 1 : self._pointer]
-        handle = make_byte_string(handle)
         return unpack_bitstring(handle)
 
     def decode_16bit_uint(self):
@@ -391,14 +373,12 @@ class BinaryPayloadDecoder:
         self._pointer += 2
         fstring = self._byteorder + "H"
         handle = self._payload[self._pointer - 2 : self._pointer]
-        handle = make_byte_string(handle)
         return unpack(fstring, handle)[0]
 
     def decode_32bit_uint(self):
         """Decode a 32 bit unsigned int from the buffer."""
         self._pointer += 4
         fstring = "I"
-        # fstring = "I"
         handle = self._payload[self._pointer - 4 : self._pointer]
         handle = self._unpack_words(fstring, handle)
         return unpack("!" + fstring, handle)[0]
@@ -416,7 +396,6 @@ class BinaryPayloadDecoder:
         self._pointer += 1
         fstring = self._byteorder + "b"
         handle = self._payload[self._pointer - 1 : self._pointer]
-        handle = make_byte_string(handle)
         return unpack(fstring, handle)[0]
 
     def decode_16bit_int(self):
@@ -424,7 +403,6 @@ class BinaryPayloadDecoder:
         self._pointer += 2
         fstring = self._byteorder + "h"
         handle = self._payload[self._pointer - 2 : self._pointer]
-        handle = make_byte_string(handle)
         return unpack(fstring, handle)[0]
 
     def decode_32bit_int(self):
@@ -481,9 +459,3 @@ class BinaryPayloadDecoder:
         :param nbytes: The number of bytes to skip
         """
         self._pointer += nbytes
-
-
-# ---------------------------------------------------------------------------#
-#  Exported Identifiers
-# ---------------------------------------------------------------------------#
-__all__ = ["BinaryPayloadBuilder", "BinaryPayloadDecoder"]
