@@ -1,25 +1,9 @@
 """Pymodbus REPL Entry point."""
-# pylint: disable=anomalous-backslash-in-string
-# flake8: noqa: W605
 import logging
 import pathlib
-import sys
-
-
-try:
-    import click
-except ImportError:
-    print('click not installed!! Install with "pip install click"')
-    sys.exit(1)
-try:
-    from prompt_toolkit import PromptSession, print_formatted_text
-except ImportError:
-    print(
-        "prompt toolkit is not installed!! "
-        'Install with "pip install prompt_toolkit --upgrade"'
-    )
-    sys.exit(1)
 
+import click
+from prompt_toolkit import PromptSession, print_formatted_text
 from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
 from prompt_toolkit.formatted_text import HTML
 from prompt_toolkit.history import FileHistory
@@ -28,8 +12,12 @@ from prompt_toolkit.lexers import PygmentsLexer
 from prompt_toolkit.styles import Style
 from pygments.lexers.python import PythonLexer
 
+from pymodbus import __version__ as pymodbus_version
 from pymodbus.exceptions import ParameterException
-from pymodbus.repl.client.completer import CmdCompleter, has_selected_completion
+from pymodbus.repl.client.completer import (
+    CmdCompleter,
+    has_selected_completion,
+)
 from pymodbus.repl.client.helper import CLIENT_ATTRIBUTES, Result
 from pymodbus.repl.client.mclient import ModbusSerialClient, ModbusTcpClient
 from pymodbus.transaction import (
@@ -38,14 +26,11 @@ from pymodbus.transaction import (
     ModbusRtuFramer,
     ModbusSocketFramer,
 )
-from pymodbus.version import version
 
 
 _logger = logging.getLogger()
 
-click.disable_unicode_literals_warning = True
-
-TITLE = f"""
+TITLE = rf"""
 ----------------------------------------------------------------------------
 __________          _____             .___  __________              .__
 \______   \___.__. /     \   ____   __| _/  \______   \ ____ ______ |  |
@@ -53,7 +38,7 @@ __________          _____             .___  __________              .__
  |    |    \___  /    Y    (  <_> ) /_/ |    |    |   \  ___/|  |_> >  |__
  |____|    / ____\____|__  /\____/\____ | /\ |____|_  /\___  >   __/|____/
            \/            \/            \/ \/        \/     \/|__|
-                                        v1.3.0 - {version}
+                                        v1.3.0 - {pymodbus_version}
 ----------------------------------------------------------------------------
 """
 
@@ -110,19 +95,15 @@ class NumericChoice(click.Choice):
                     return choice
 
         self.fail(
-            "invalid choice: %s. (choose from %s)"  # pylint: disable=consider-using-f-string
-            % (
-                value,
-                ", ".join(self.choices),
-            ),
+            f"invalid choice: {value}. (choose from {', '.join(self.choices)})",
             param,
             ctx,
         )
         return None
 
 
-def _process_args(args: list, string: bool=True):
-    """Internal function to parse arguments provided on command line.
+def _process_args(args: list, string: bool = True):
+    """Parse arguments provided on command line.
 
     :param args: Array of argument values
     :param string: True if arguments values are strings, false if argument values are integers
@@ -166,94 +147,107 @@ def _process_args(args: list, string: bool=True):
     return kwargs, execute
 
 
-def cli(client):  # noqa: C901 pylint: disable=too-complex
-    """Run client definition."""
-    use_keys = KeyBindings()
-    history_file = pathlib.Path.home().joinpath(".pymodhis")
+class CLI:  # pylint: disable=too-few-public-methods
+    """Client definition."""
 
-    @use_keys.add("c-space")
-    def _(event):
-        """Initialize autocompletion, or select the next completion."""
-        buff = event.app.current_buffer
-        if buff.complete_state:
-            buff.complete_next()
-        else:
-            buff.start_completion(select_first=False)
+    def __init__(self, client):
+        """Set up client and keybindings."""
+        use_keys = KeyBindings()
+        history_file = pathlib.Path.home().joinpath(".pymodhis")
+        self.client = client
 
-    @use_keys.add("enter", filter=has_selected_completion)
-    def _(event):
-        """Make the enter key work as the tab key only when showing the menu."""
-        event.current_buffer.complete_state = None
-        buffer = event.cli.current_buffer
-        buffer.complete_state = None
+        @use_keys.add("c-space")
+        def _(event):
+            """Initialize autocompletion, or select the next completion."""
+            buff = event.app.current_buffer
+            if buff.complete_state:
+                buff.complete_next()
+            else:
+                buff.start_completion(select_first=False)
 
-    session = PromptSession(
-        lexer=PygmentsLexer(PythonLexer),
-        completer=CmdCompleter(client),
-        style=style,
-        complete_while_typing=True,
-        bottom_toolbar=bottom_toolbar,
-        key_bindings=use_keys,
-        history=FileHistory(history_file),
-        auto_suggest=AutoSuggestFromHistory(),
-    )
-    click.secho(TITLE, fg="green")
-    result = None
-    while True:  # pylint: disable=too-many-nested-blocks
-        try:
+        @use_keys.add("enter", filter=has_selected_completion)
+        def _(event):
+            """Make the enter key work as the tab key only when showing the menu."""
+            event.current_buffer.complete_state = None
+            buffer = event.cli.current_buffer
+            buffer.complete_state = None
 
-            text = session.prompt("> ", complete_while_typing=True)
-            if text.strip().lower() == "help":
-                print_formatted_text(HTML("<u>Available commands:</u>"))
-                for cmd, obj in sorted(session.completer.commands.items()):
-                    if cmd != "help":
-                        print_formatted_text(
-                            HTML(
-                                "<skyblue>{:45s}</skyblue>"  # pylint: disable=consider-using-f-string
-                                "<seagreen>{:100s}"
-                                "</seagreen>".format(cmd, obj.help_text)
-                            )
-                        )
+        self.session = PromptSession(
+            lexer=PygmentsLexer(PythonLexer),
+            completer=CmdCompleter(client),
+            style=style,
+            complete_while_typing=True,
+            bottom_toolbar=bottom_toolbar,
+            key_bindings=use_keys,
+            history=FileHistory(history_file),
+            auto_suggest=AutoSuggestFromHistory(),
+        )
+        click.secho(TITLE, fg="green")
 
-                continue
-            if text.strip().lower() == "exit":
-                raise EOFError()
-            if text.strip().lower().startswith("client."):
-                try:
-                    text = text.strip().split()
-                    cmd = text[0].split(".")[1]
-                    args = text[1:]
-                    kwargs, execute = _process_args(args, string=False)
-                    if execute:
-                        if text[0] in CLIENT_ATTRIBUTES:
-                            result = Result(getattr(client, cmd))
-                        else:
-                            result = Result(getattr(client, cmd)(**kwargs))
-                        result.print_result()
-                except Exception as exc:  # pylint: disable=broad-except
-                    click.secho(repr(exc), fg="red")
-            elif text.strip().lower().startswith("result."):
-                if result:
-                    words = text.lower().split()
-                    if words[0] == "result.raw":
-                        result.raw()
-                    if words[0] == "result.decode":
-                        args = words[1:]
-                        kwargs, execute = _process_args(args)
-                        if execute:
-                            result.decode(**kwargs)
-        except KeyboardInterrupt:
-            continue  # Control-C pressed. Try again.
-        except EOFError:
-            break  # Control-D pressed.
-        except Exception as exc:  # Handle all other exceptions pylint: disable=broad-except
-            click.secho(str(exc), fg="red")
+    def _print_command_help(self, commands):
+        """Print a list of commands with help text."""
+        for cmd, obj in sorted(commands.items()):
+            if cmd != "help":
+                print_formatted_text(
+                    HTML(
+                        f"<skyblue>{cmd:45s}</skyblue>"
+                        f"<seagreen>{obj.help_text:100s}"
+                        "</seagreen>"
+                    )
+                )
 
-    click.secho("GoodBye!", fg="blue")
+    def _process_client(self, text, client) -> Result:
+        """Process client commands."""
+        text = text.strip().split()
+        cmd = text[0].split(".")[1]
+        args = text[1:]
+        kwargs, execute = _process_args(args, string=False)
+        if execute:
+            if text[0] in CLIENT_ATTRIBUTES:
+                result = Result(getattr(client, cmd))
+            else:
+                result = Result(getattr(client, cmd)(**kwargs))
+            result.print_result()
+        return result
+
+    def _process_result(self, text, result):
+        """Process result commands."""
+        words = text.split()
+        if words[0] == "result.raw":
+            result.raw()
+        if words[0] == "result.decode":
+            args = words[1:]
+            kwargs, execute = _process_args(args)
+            if execute:
+                result.decode(**kwargs)
+
+    def run(self):
+        """Run the REPL."""
+        result = None
+        while True:
+            try:
+                text = self.session.prompt("> ", complete_while_typing=True)
+                if text.strip().lower() == "help":
+                    print_formatted_text(HTML("<u>Available commands:</u>"))
+                    self._print_command_help(self.session.completer.commands)
+                elif text.strip().lower() == "exit":
+                    raise EOFError()
+                elif text.strip().lower().startswith("client."):
+                    result = self._process_client(text, self.client)
+                elif text.strip().lower().startswith("result.") and result:
+                    self._process_result(text, result)
+            except KeyboardInterrupt:
+                continue  # Control-C pressed. Try again.
+            except EOFError:
+                break  # Control-D pressed.
+            except Exception as exc:  # pylint: disable=broad-except
+                click.secho(str(exc), fg="red")
+
+        click.secho("GoodBye!", fg="blue")
 
 
 @click.group("pymodbus-repl")
-@click.version_option(version, message=TITLE)
+@click.version_option(str(pymodbus_version), message=TITLE)
 @click.option("--verbose", is_flag=True, default=False, help="Verbose logs")
 @click.option(
     "--broadcast-support",
@@ -268,11 +262,6 @@ def cli(client):  # noqa: C901 pylint: disable=too-complex
     "--retry-on-error", is_flag=True, default=False, help="Retry on error response"
 )
 @click.option("--retries", default=3, help="Retry count")
-@click.option(
-    "--reset-socket/--no-reset-socket",
-    default=True,
-    help="Reset client socket on error",
-)
 @click.pass_context
 def main(
     ctx,
@@ -281,7 +270,6 @@ def main(
     retry_on_empty,
     retry_on_error,
     retries,
-    reset_socket,
 ):
     """Run Main."""
     if verbose:
@@ -292,11 +280,10 @@ def main(
         logging.basicConfig(format=use_format)
         _logger.setLevel(logging.DEBUG)
     ctx.obj = {
-        "broadcast": broadcast_support,
+        "broadcast_enable": broadcast_support,
         "retry_on_empty": retry_on_empty,
         "retry_on_invalid": retry_on_error,
         "retries": retries,
-        "reset_socket": reset_socket,
     }
 
 
@@ -322,7 +309,8 @@ def tcp(ctx, host, port, framer):
     if framer == "rtu":
         kwargs["framer"] = ModbusRtuFramer
     client = ModbusTcpClient(**kwargs)
-    cli(client)
+    cli = CLI(client)
+    cli.run()
 
 
 @main.command("serial")
@@ -341,7 +329,7 @@ def tcp(ctx, host, port, framer):
 )
 @click.option(
     "--baudrate",
-    help="Modbus RTU serial baudrate to use. Defaults to 9600",
+    help="Modbus RTU serial baudrate to use.",
     default=9600,
     type=int,
 )
@@ -349,7 +337,7 @@ def tcp(ctx, host, port, framer):
     "--bytesize",
     help="Modbus RTU serial Number of data bits. "
     "Possible values: FIVEBITS, SIXBITS, SEVENBITS, "
-    "EIGHTBITS. Defaults to 8",
+    "EIGHTBITS.",
     type=NumericChoice(["5", "6", "7", "8"], int),
     default="8",
 )
@@ -372,33 +360,31 @@ def tcp(ctx, host, port, framer):
 )
 @click.option(
     "--xonxoff",
-    help="Modbus RTU serial xonxoff.  Enable software flow control. Defaults to 0",
+    help="Modbus RTU serial xonxoff.  Enable software flow control.",
     default=0,
     type=int,
 )
 @click.option(
     "--rtscts",
-    help="Modbus RTU serial rtscts. Enable hardware (RTS/CTS) flow "
-    "control. Defaults to 0",
+    help="Modbus RTU serial rtscts. Enable hardware (RTS/CTS) flow " "control.",
     default=0,
     type=int,
 )
 @click.option(
     "--dsrdtr",
-    help="Modbus RTU serial dsrdtr. Enable hardware (DSR/DTR) flow "
-    "control. Defaults to 0",
+    help="Modbus RTU serial dsrdtr. Enable hardware (DSR/DTR) flow " "control.",
     default=0,
     type=int,
 )
 @click.option(
     "--timeout",
-    help="Modbus RTU serial read timeout. Defaults to 0.025 sec",
+    help="Modbus RTU serial read timeout.",
     default=0.25,
     type=float,
 )
 @click.option(
     "--write-timeout",
-    help="Modbus RTU serial write timeout. Defaults to 2 sec",
+    help="Modbus RTU serial write timeout.",
     default=2,
     type=float,
 )
@@ -442,7 +428,8 @@ def serial(  # pylint: disable=too-many-arguments
         write_timeout=write_timeout,
         **ctx.obj,
     )
-    cli(client)
+    cli = CLI(client)
+    cli.run()
 
 
 if __name__ == "__main__":
