@@ -2,11 +2,8 @@
 # pylint: disable=missing-type-doc
 import functools
 
-from pymodbus.client import (
-    ModbusSerialClient as _ModbusSerialClient,
-    ModbusTcpClient as _ModbusTcpClient,
-)
-from pymodbus.constants import Defaults
+from pymodbus.client import ModbusSerialClient as _ModbusSerialClient
+from pymodbus.client import ModbusTcpClient as _ModbusTcpClient
 from pymodbus.diag_message import (
     ChangeAsciiInputDelimiterRequest,
     ClearCountersRequest,
@@ -56,7 +53,7 @@ def handle_brodcast(func):
     def _wrapper(*args, **kwargs):
         self = args[0]
         resp = func(*args, **kwargs)
-        if not kwargs.get("unit") and self.params.broadcast_enable:
+        if not kwargs.get("slave") and self.params.broadcast_enable:
             return {"broadcasted": True}
         if not resp.isError():
             return make_response_dict(resp)
@@ -73,8 +70,7 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
     @staticmethod
     def _process_exception(resp, **kwargs):
         """Set internal process exception."""
-        unit = kwargs.get("unit")
-        if unit == 0:  # pylint: disable=compare-to-zero,disable=consider-using-assignment-expr
+        if "slave" not in kwargs:
             err = {"message": "Broadcast message, ignoring errors!!!"}
         else:
             if isinstance(resp, ExceptionResponse):  # pylint: disable=else-if-used
@@ -93,12 +89,12 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
                 err = {"error": str(resp)}
         return err
 
-    def read_coils(self, address, count=1, slave=Defaults.Slave, **kwargs):
+    def read_coils(self, address, count=1, slave=0, **kwargs):
         """Read `count` coils from a given slave starting at `address`.
 
         :param address: The starting address to read from
         :param count: The number of coils to read
-        :param slave: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         :param kwargs:
         :returns: List of register values
         """
@@ -107,14 +103,14 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
         )
         if not resp.isError():
             return {"function_code": resp.function_code, "bits": resp.bits}
-        return ExtendedRequestSupport._process_exception(resp)
+        return ExtendedRequestSupport._process_exception(resp, slave=slave)
 
-    def read_discrete_inputs(self, address, count=1, slave=Defaults.Slave, **kwargs):
+    def read_discrete_inputs(self, address, count=1, slave=0, **kwargs):
         """Read `count` number of discrete inputs starting at offset `address`.
 
         :param address: The starting address to read from
         :param count: The number of coils to read
-        :param slave: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         :param kwargs:
         :return: List of bits
         """
@@ -123,15 +119,15 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
         )
         if not resp.isError():
             return {"function_code": resp.function_code, "bits": resp.bits}
-        return ExtendedRequestSupport._process_exception(resp)
+        return ExtendedRequestSupport._process_exception(resp, slave=slave)
 
     @handle_brodcast
-    def write_coil(self, address, value, slave=Defaults.Slave, **kwargs):
+    def write_coil(self, address, value, slave=0, **kwargs):
         """Write `value` to coil at `address`.
 
         :param address: coil offset to write to
         :param value: bit value to write
-        :param slave: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         :param kwargs:
         :return:
         """
@@ -141,12 +137,12 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
         return resp
 
     @handle_brodcast
-    def write_coils(self, address, values, slave=Defaults.Slave, **kwargs):
+    def write_coils(self, address, values, slave=0, **kwargs):
         """Write `value` to coil at `address`.
 
         :param address: coil offset to write to
         :param values: list of bit values to write (comma separated)
-        :param slave: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         :param kwargs:
         :return:
         """
@@ -156,12 +152,12 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
         return resp
 
     @handle_brodcast
-    def write_register(self, address, value, slave=Defaults.Slave, **kwargs):
+    def write_register(self, address, value, slave=0, **kwargs):
         """Write `value` to register at `address`.
 
         :param address: register offset to write to
         :param value: register value to write
-        :param slave: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         :param kwargs:
         :return:
         """
@@ -171,12 +167,12 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
         return resp
 
     @handle_brodcast
-    def write_registers(self, address, values, slave=Defaults.Slave, **kwargs):
+    def write_registers(self, address, values, slave=0, **kwargs):
         """Write list of `values` to registers starting at `address`.
 
         :param address: register offset to write to
         :param values: list of register value to write (comma separated)
-        :param slave: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         :param kwargs:
         :return:
         """
@@ -185,12 +181,12 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
         )
         return resp
 
-    def read_holding_registers(self, address, count=1, slave=Defaults.Slave, **kwargs):
+    def read_holding_registers(self, address, count=1, slave=0, **kwargs):
         """Read `count` number of holding registers starting at `address`.
 
         :param address: starting register offset to read from
         :param count: Number of registers to read
-        :param slave: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         :param kwargs:
         :return:
         """
@@ -199,14 +195,14 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
         )
         if not resp.isError():
             return {"function_code": resp.function_code, "registers": resp.registers}
-        return ExtendedRequestSupport._process_exception(resp)
+        return ExtendedRequestSupport._process_exception(resp, slave=slave)
 
-    def read_input_registers(self, address, count=1, slave=Defaults.Slave, **kwargs):
+    def read_input_registers(self, address, count=1, slave=0, **kwargs):
         """Read `count` number of input registers starting at `address`.
 
         :param address: starting register offset to read from to
         :param count: Number of registers to read
-        :param slave: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         :param kwargs:
         :return:
         """
@@ -215,15 +211,15 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
         )
         if not resp.isError():
             return {"function_code": resp.function_code, "registers": resp.registers}
-        return ExtendedRequestSupport._process_exception(resp)
+        return ExtendedRequestSupport._process_exception(resp, slave=slave)
 
     def readwrite_registers(
         self,
-        read_address,
-        read_count,
-        write_address,
-        write_registers,
-        unit=Defaults.Slave,
+        read_address=0,
+        read_count=0,
+        write_address=0,
+        values=0,
+        slave=0,
         **kwargs,
     ):
         """Read `read_count` number of holding registers.
@@ -234,8 +230,8 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
         :param read_address: register offset to read from
         :param read_count: Number of registers to read
         :param write_address: register offset to write to
-        :param write_registers: List of register values to write (comma separated)
-        :param unit: Modbus slave unit ID
+        :param values: List of register values to write (comma separated)
+        :param slave: Modbus slave slave ID
         :param kwargs:
         :return:
         """
@@ -243,20 +239,20 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
             read_address=read_address,
             read_count=read_count,
             write_address=write_address,
-            write_registers=write_registers,
-            unit=unit,
+            values=values,
+            slave=slave,
             **kwargs,
         )
         if not resp.isError():
             return {"function_code": resp.function_code, "registers": resp.registers}
-        return ExtendedRequestSupport._process_exception(resp)
+        return ExtendedRequestSupport._process_exception(resp, slave=slave)
 
     def mask_write_register(
         self,
         address=0x0000,
         and_mask=0xFFFF,
         or_mask=0x0000,
-        unit=Defaults.Slave,
+        slave=0,
         **kwargs,
     ):
         """Mask content of holding register at `address` with `and_mask` and `or_mask`.
@@ -264,12 +260,12 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
         :param address: Reference address of register
         :param and_mask: And Mask
         :param or_mask: OR Mask
-        :param unit: Modbus slave unit ID
+        :param slave: Modbus slave slave ID
         :param kwargs:
         :return:
         """
         resp = super().mask_write_register(  # pylint: disable=no-member
-            address=address, and_mask=and_mask, or_mask=or_mask, unit=unit, **kwargs
+            address=address, and_mask=and_mask, or_mask=or_mask, slave=slave, **kwargs
         )
         if not resp.isError():
             return {
@@ -278,7 +274,7 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
                 "and mask": resp.and_mask,
                 "or mask": resp.or_mask,
             }
-        return ExtendedRequestSupport._process_exception(resp)
+        return ExtendedRequestSupport._process_exception(resp, slave=slave)
 
     def read_device_information(self, read_code=None, object_id=0x00, **kwargs):
         """Read the identification and additional information of remote slave.
@@ -300,16 +296,16 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
                 "more follows": resp.more_follows,
                 "space left": resp.space_left,
             }
-        return ExtendedRequestSupport._process_exception(resp)
+        return ExtendedRequestSupport._process_exception(resp, slave=request.slave_id)
 
-    def report_slave_id(self, unit=Defaults.Slave, **kwargs):
+    def report_slave_id(self, slave=0, **kwargs):
         """Report information about remote slave ID.
 
-        :param unit: Modbus slave unit ID
+        :param slave: Modbus slave ID
         :param kwargs:
         :return:
         """
-        request = ReportSlaveIdRequest(unit, **kwargs)
+        request = ReportSlaveIdRequest(slave, **kwargs)
         resp = self.execute(request)  # pylint: disable=no-member
         if not resp.isError():
             return {
@@ -318,26 +314,25 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
                 "status": resp.status,
                 "byte count": resp.byte_count,
             }
-        return ExtendedRequestSupport._process_exception(resp)
+        return ExtendedRequestSupport._process_exception(resp, slave=slave)
 
-    def read_exception_status(self, **kwargs):
-        """Read tcontents of eight Exception Status output.
-
-        In a remote device.
+    def read_exception_status(self, slave=0, **kwargs):
+        """Read contents of eight Exception Status output in a remote device.
 
+        :param slave: Modbus slave ID
         :param kwargs:
         :return:
         """
-        request = ReadExceptionStatusRequest(**kwargs)
+        request = ReadExceptionStatusRequest(slave, **kwargs)
         resp = self.execute(request)  # pylint: disable=no-member
         if not resp.isError():
             return {"function_code": resp.function_code, "status": resp.status}
-        return ExtendedRequestSupport._process_exception(resp)
+        return ExtendedRequestSupport._process_exception(resp, slave=request.slave_id)
 
     def get_com_event_counter(self, **kwargs):
         """Read status word and an event count.
 
-        From the remote device"s communication event counter.
+        From the remote device's communication event counter.
 
         :param kwargs:
         :return:
@@ -350,7 +345,7 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
                 "status": resp.status,
                 "count": resp.count,
             }
-        return ExtendedRequestSupport._process_exception(resp)
+        return ExtendedRequestSupport._process_exception(resp, slave=request.slave_id)
 
     def get_com_event_log(self, **kwargs):
         """Read status word.
@@ -371,7 +366,7 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
                 "event count": resp.event_count,
                 "events": resp.events,
             }
-        return ExtendedRequestSupport._process_exception(resp)
+        return ExtendedRequestSupport._process_exception(resp, slave=request.slave_id)
 
     def _execute_diagnostic_request(self, request):
         """Execute diagnostic request."""
@@ -382,7 +377,7 @@ class ExtendedRequestSupport:  # pylint: disable=(too-many-public-methods
                 "sub function code": resp.sub_function_code,
                 "message": resp.message,
             }
-        return ExtendedRequestSupport._process_exception(resp)
+        return ExtendedRequestSupport._process_exception(resp, slave=request.slave_id)
 
     def return_query_data(self, message=0, **kwargs):
         """Loop back data sent in response.
@@ -571,14 +566,14 @@ class ModbusSerialClient(ExtendedRequestSupport, _ModbusSerialClient):
 
         :return: Current Serial port
         """
-        return self.params.port
+        return self.comm_params.port
 
     def set_port(self, value):
         """Set serial Port setter.
 
         :param value: New port
         """
-        self.params.port = value
+        self.comm_params.port = value
         if self.is_socket_open():
             self.close()
 
@@ -603,7 +598,7 @@ class ModbusSerialClient(ExtendedRequestSupport, _ModbusSerialClient):
 
         :return: Current bytesize
         """
-        return self.params.bytesize
+        return self.comm_params.bytesize
 
     def set_bytesize(self, value):
         """Set Byte size.
@@ -611,7 +606,7 @@ class ModbusSerialClient(ExtendedRequestSupport, _ModbusSerialClient):
         :param value: Possible values (5, 6, 7, 8)
 
         """
-        self.params.bytesize = int(value)
+        self.comm_params.bytesize = int(value)
         if self.is_socket_open():
             self.close()
 
@@ -636,14 +631,14 @@ class ModbusSerialClient(ExtendedRequestSupport, _ModbusSerialClient):
 
         :return: Current baudrate
         """
-        return self.params.baudrate
+        return self.comm_params.baudrate
 
     def set_baudrate(self, value):
         """Set baudrate setter.
 
         :param value: <supported baudrate>
         """
-        self.params.baudrate = int(value)
+        self.comm_params.baudrate = int(value)
         if self.is_socket_open():
             self.close()
 
@@ -652,14 +647,14 @@ class ModbusSerialClient(ExtendedRequestSupport, _ModbusSerialClient):
 
         :return: Current read imeout.
         """
-        return self.params.timeout
+        return self.comm_params.timeout_connect
 
     def set_timeout(self, value):
         """Read timeout setter.
 
         :param value: Read Timeout in seconds
         """
-        self.params.timeout = float(value)
+        self.comm_params.timeout_connect = float(value)
         if self.is_socket_open():
             self.close()
 
@@ -669,12 +664,12 @@ class ModbusSerialClient(ExtendedRequestSupport, _ModbusSerialClient):
         :return: Current Serial settings as dict.
         """
         return {
-            "baudrate": self.params.baudrate,
-            "port": self.params.port,
-            "parity": self.params.parity,
-            "stopbits": self.params.stopbits,
-            "bytesize": self.params.bytesize,
-            "read timeout": self.params.timeout,
+            "baudrate": self.comm_params.baudrate,
+            "port": self.comm_params.port,
+            "parity": self.comm_params.parity,
+            "stopbits": self.comm_params.stopbits,
+            "bytesize": self.comm_params.bytesize,
+            "read timeout": self.comm_params.timeout_connect,
             "t1.5": self.inter_char_timeout,
             "t3.5": self.silent_interval,
         }
