@@ -2,30 +2,30 @@
 from __future__ import annotations
 
 import asyncio
-from enum import Enum
+import contextlib
 import json
 import logging
+import sys
+from enum import Enum
 from pathlib import Path
-from typing import List
 
 import typer
 
+from pymodbus import pymodbus_apply_logging_config
 from pymodbus.framer.socket_framer import ModbusSocketFramer
+from pymodbus.logging import Log
 from pymodbus.repl.server.cli import run_repl
 from pymodbus.server.reactive.default_config import DEFAULT_CONFIG
 from pymodbus.server.reactive.main import (
     DEFAULT_FRAMER,
-    DEFUALT_HANDLERS,
     ReactiveServer,
 )
 
 
 CANCELLED_ERROR = asyncio.exceptions.CancelledError
-
-_logger = logging.getLogger(__name__)
-
 CONTEXT_SETTING = {"allow_extra_args": True, "ignore_unknown_options": True}
 
+
 # TBD class ModbusServerConfig:
 
 
@@ -50,7 +50,7 @@ class ModbusFramerTypes(str, Enum):
     binary = "binary"  # pylint: disable=invalid-name
 
 
-def _completer(incomplete: str, valid_values: List[str]) -> List[str]:
+def _completer(incomplete: str, valid_values: list[str]) -> list[str]:
     """Complete value."""
     completion = []
     for name in valid_values:
@@ -59,22 +59,38 @@ def _completer(incomplete: str, valid_values: List[str]) -> List[str]:
     return completion
 
 
-def framers(incomplete: str) -> List[str]:
+def framers(incomplete: str) -> list[str]:
     """Return an autocompleted list of supported clouds."""
     _framers = ["socket", "rtu", "tls", "ascii", "binary"]
     return _completer(incomplete, _framers)
 
 
-def servers(incomplete: str) -> List[str]:
+def servers(incomplete: str) -> list[str]:
     """Return an autocompleted list of supported clouds."""
     _servers = ["tcp", "serial", "tls", "udp"]
     return _completer(incomplete, _servers)
 
 
+def process_extra_args(extra_args: list[str], modbus_config: dict) -> dict:
+    """Process extra args passed to server."""
+    options_stripped = [x.strip().replace("--", "") for x in extra_args[::2]]
+    extra_args_dict = dict(list(zip(options_stripped, extra_args[1::2])))
+    for option, value in extra_args_dict.items():
+        if option in modbus_config:
+            try:
+                modbus_config[option] = type(modbus_config[option])(value)
+            except ValueError as err:
+                Log.error(
+                    "Error parsing extra arg {} with value '{}'. {}", option, value, err
+                )
+                sys.exit(1)
+    return modbus_config
+
+
 app = typer.Typer(
     no_args_is_help=True,
     context_settings=CONTEXT_SETTING,
-    help="Reactive modebus server",
+    help="Reactive Modbus server",
 )
 
 
@@ -92,25 +108,14 @@ def server(
     ),
 ):
     """Run server code."""
-    FORMAT = (  # pylint: disable=invalid-name
-        "%(asctime)-15s %(threadName)-15s"
-        " %(levelname)-8s %(module)-15s:%(lineno)-8s %(message)s"
-    )
-    pymodbus_logger = logging.getLogger("pymodbus")
-    logging.basicConfig(format=FORMAT)
-    logger = logging.getLogger(__name__)
-    if verbose:
-        pymodbus_logger.setLevel(logging.DEBUG)
-        logger.setLevel(logging.DEBUG)
-    else:
-        pymodbus_logger.setLevel(logging.ERROR)
-        logger.setLevel(logging.ERROR)
+    log_level = logging.DEBUG if verbose else logging.ERROR
+    pymodbus_apply_logging_config(log_level)
 
     ctx.obj = {
         "repl": repl,
         "host": host,
         "web_port": web_port,
-        "broadcast": broadcast_support,
+        "broadcast_enable": broadcast_support,
     }
 
 
@@ -118,7 +123,7 @@ def server(
 def run(
     ctx: typer.Context,
     modbus_server: str = typer.Option(
-        ModbusServerTypes.tcp,
+        ModbusServerTypes.tcp.value,
         "--modbus-server",
         "-s",
         case_sensitive=False,
@@ -126,7 +131,7 @@ def run(
         help="Modbus Server",
     ),
     modbus_framer: str = typer.Option(
-        ModbusFramerTypes.socket,
+        ModbusFramerTypes.socket.value,
         "--framer",
         "-f",
         case_sensitive=False,
@@ -134,10 +139,10 @@ def run(
         help="Modbus framer to use",
     ),
     modbus_port: str = typer.Option("5020", "--modbus-port", "-p", help="Modbus port"),
-    modbus_unit_id: List[int] = typer.Option(
-        None, "--unit-id", "-u", help="Supported Modbus unit id's"
+    modbus_slave_id: list[int] = typer.Option(
+        [1], "--slave-id", "-u", help="Supported Modbus slave id's"
     ),
-    modbus_config: Path = typer.Option(
+    modbus_config_path: Path = typer.Option(
         None, help="Path to additional modbus server config"
     ),
     randomize: int = typer.Option(
@@ -147,6 +152,13 @@ def run(
         help="Randomize every `r` reads. 0=never, 1=always,2=every-second-read"
         ", and so on. Applicable IR and DI.",
     ),
+    change_rate: int = typer.Option(
+        0,
+        "--change-rate",
+        "-c",
+        help="Rate in % registers to change. 0=none, 100=all, 12=12% of registers"
+        ", and so on. Applicable IR and DI.",
+    ),
 ):
     """Run Reactive Modbus server.
 
@@ -157,44 +169,41 @@ def run(
     web_app_config = ctx.obj
     loop = asyncio.get_event_loop()
     framer = DEFAULT_FRAMER.get(modbus_framer, ModbusSocketFramer)
-    if modbus_config:
-        with open(modbus_config) as my_file:  # pylint: disable=unspecified-encoding
+    if modbus_config_path:
+        with open(modbus_config_path, encoding="utf-8") as my_file:
             modbus_config = json.load(my_file)
     else:
         modbus_config = DEFAULT_CONFIG
 
+    extra_args = ctx.args
+    data_block_settings = modbus_config.pop("data_block_settings", {})
     modbus_config = modbus_config.get(modbus_server, {})
+    modbus_config = process_extra_args(extra_args, modbus_config)
 
-    if modbus_server != "serial":
-        modbus_port = int(modbus_port)
-        handler = modbus_config.pop("handler", "ModbusConnectedRequestHandler")
-    else:
-        handler = modbus_config.pop("handler", "ModbusSingleRequestHandler")
-    handler = DEFUALT_HANDLERS.get(handler.strip())
-
-    modbus_config["handler"] = handler
     modbus_config["randomize"] = randomize
-    app = ReactiveServer.factory(
-        modbus_server,
-        framer,
-        modbus_port=modbus_port,
-        unit=modbus_unit_id,
-        loop=loop,
-        single=False,
-        **web_app_config,
-        **modbus_config,
-    )
-    try:
-        if repl:
-            loop.run_until_complete(app.run_async())
+    modbus_config["change_rate"] = change_rate
 
+    async def _wrapper():
+        app = ReactiveServer.factory(
+            modbus_server,
+            framer,
+            modbus_port=modbus_port,
+            slave=modbus_slave_id,
+            single=False,
+            data_block_settings=data_block_settings,
+            **web_app_config,
+            **modbus_config,
+        )
+        await app.run_async(repl)
+        return app
+
+    app = loop.run_until_complete(_wrapper())
+    if repl:
+        with contextlib.suppress(asyncio.CancelledError):
             loop.run_until_complete(run_repl(app))
-            loop.run_forever()
-        else:
-            app.run()
 
-    except CANCELLED_ERROR:
-        print("Done!!!!!")
+    else:
+        loop.run_forever()
 
 
 if __name__ == "__main__":
