@@ -1,13 +1,21 @@
 """Collection of transaction based abstractions."""
-# pylint: disable=missing-type-doc
-from functools import partial
-import logging
-import socket
-import struct
-from threading import RLock
-import time
 
-from pymodbus.constants import Defaults
+__all__ = [
+    "DictTransactionManager",
+    "ModbusSocketFramer",
+    "ModbusTlsFramer",
+    "ModbusRtuFramer",
+    "ModbusAsciiFramer",
+    "ModbusBinaryFramer",
+]
+
+# pylint: disable=missing-type-doc
+import struct
+import time
+from contextlib import suppress
+from functools import partial
+from threading import RLock
+
 from pymodbus.exceptions import (
     InvalidMessageReceivedException,
     ModbusIOException,
@@ -18,15 +26,10 @@ from pymodbus.framer.binary_framer import ModbusBinaryFramer
 from pymodbus.framer.rtu_framer import ModbusRtuFramer
 from pymodbus.framer.socket_framer import ModbusSocketFramer
 from pymodbus.framer.tls_framer import ModbusTlsFramer
+from pymodbus.logging import Log
 from pymodbus.utilities import ModbusTransactionState, hexlify_packets
 
 
-# --------------------------------------------------------------------------- #
-# Logging
-# --------------------------------------------------------------------------- #
-_logger = logging.getLogger(__name__)
-
-
 # --------------------------------------------------------------------------- #
 # The Global Transaction Manager
 # --------------------------------------------------------------------------- #
@@ -53,13 +56,12 @@ class ModbusTransactionManager:
         :param retry_on_empty: Should the client retry on empty
         :param retries: The number of retries to allow
         """
-        self.tid = Defaults.TransactionId
+        self.tid = 0
         self.client = client
-        self.backoff = kwargs.get("backoff", Defaults.Backoff) or 0.3
-        self.retry_on_empty = kwargs.get("retry_on_empty", Defaults.RetryOnEmpty)
-        self.retry_on_invalid = kwargs.get("retry_on_invalid", Defaults.RetryOnInvalid)
-        self.retries = kwargs.get("retries", Defaults.Retries) or 1
-        self.reset_socket = kwargs.get("reset_socket", True)
+        self.backoff = kwargs.get("backoff", 0.3)
+        self.retry_on_empty = kwargs.get("retry_on_empty", False)
+        self.retry_on_invalid = kwargs.get("retry_on_invalid", False)
+        self.retries = kwargs.get("retries", 3)
         self._transaction_lock = RLock()
         self._no_response_devices = []
         if client:
@@ -97,7 +99,7 @@ class ModbusTransactionManager:
             return self.base_adu_size + 2  # Fcode(1), ExceptionCode(1)
         return None
 
-    def _validate_response(self, request, response, exp_resp_len):
+    def _validate_response(self, request, response, exp_resp_len, is_udp=False):
         """Validate Incoming response against request.
 
         :param request: Request sent
@@ -110,36 +112,33 @@ class ModbusTransactionManager:
 
         mbap = self.client.framer.decode_data(response)
         if (
-            mbap.get("unit") != request.unit_id
+            mbap.get("slave") != request.slave_id
             or mbap.get("fcode") & 0x7F != request.function_code
         ):
             return False
 
-        if "length" in mbap and exp_resp_len:
+        if "length" in mbap and exp_resp_len and not is_udp:
             return mbap.get("length") == exp_resp_len
         return True
 
-    def execute(self, request):  # pylint: disable=too-complex
+    def execute(self, request):  # noqa: C901
         """Start the producer to send the next request to consumer.write(Frame(request))."""
         with self._transaction_lock:
             try:
-                txt = (
-                    "Current transaction state - "
-                    f"{ModbusTransactionState.to_string(self.client.state)}"
+                Log.debug(
+                    "Current transaction state - {}",
+                    ModbusTransactionState.to_string(self.client.state),
                 )
-                _logger.debug(txt)
                 retries = self.retries
                 request.transaction_id = self.getNextTID()
-                txt = f"Running transaction {request.transaction_id}"
-                _logger.debug(txt)
+                Log.debug("Running transaction {}", request.transaction_id)
                 if _buffer := hexlify_packets(
                     self.client.framer._buffer  # pylint: disable=protected-access
                 ):
-                    txt = f"Clearing current Frame: - {_buffer}"
-                    _logger.debug(txt)
+                    Log.debug("Clearing current Frame: - {}", _buffer)
                     self.client.framer.resetFrame()
                 if broadcast := (
-                    self.client.params.broadcast_enable and not request.unit_id
+                    self.client.params.broadcast_enable and not request.slave_id
                 ):
                     self._transact(request, None, broadcast=True)
                     response = b"Broadcast write sent - no response expected"
@@ -149,22 +148,24 @@ class ModbusTransactionManager:
                         if hasattr(request, "get_response_pdu_size"):
                             response_pdu_size = request.get_response_pdu_size()
                             if isinstance(self.client.framer, ModbusAsciiFramer):
-                                response_pdu_size = response_pdu_size * 2
+                                response_pdu_size *= 2
                             if response_pdu_size:
                                 expected_response_length = (
                                     self._calculate_response_length(response_pdu_size)
                                 )
                     if (  # pylint: disable=simplifiable-if-statement
-                        request.unit_id in self._no_response_devices
+                        request.slave_id in self._no_response_devices
                     ):
                         full = True
                     else:
                         full = False
                     c_str = str(self.client)
+                    is_udp = False
                     if "modbusudpclient" in c_str.lower().strip():
+                        is_udp = True
                         full = True
                         if not expected_response_length:
-                            expected_response_length = Defaults.ReadSize
+                            expected_response_length = 1024
                     response, last_exception = self._transact(
                         request,
                         expected_response_length,
@@ -173,19 +174,20 @@ class ModbusTransactionManager:
                     )
                     while retries > 0:
                         valid_response = self._validate_response(
-                            request, response, expected_response_length
+                            request, response, expected_response_length,
+                            is_udp=is_udp
                         )
                         if valid_response:
                             if (
-                                request.unit_id in self._no_response_devices
+                                request.slave_id in self._no_response_devices
                                 and response
                             ):
-                                self._no_response_devices.remove(request.unit_id)
-                                _logger.debug("Got response!!!")
+                                self._no_response_devices.remove(request.slave_id)
+                                Log.debug("Got response!!!")
                             break
                         if not response:
-                            if request.unit_id not in self._no_response_devices:
-                                self._no_response_devices.append(request.unit_id)
+                            if request.slave_id not in self._no_response_devices:
+                                self._no_response_devices.append(request.slave_id)
                             if self.retry_on_empty:
                                 response, last_exception = self._retry_transaction(
                                     retries,
@@ -215,23 +217,25 @@ class ModbusTransactionManager:
                         tid=request.transaction_id,
                     )
                     self.client.framer.processIncomingPacket(
-                        response, addTransaction, request.unit_id
+                        response,
+                        addTransaction,
+                        request.slave_id,
+                        tid=request.transaction_id,
                     )
                     if not (response := self.getTransaction(request.transaction_id)):
                         if len(self.transactions):
                             response = self.getTransaction(tid=0)
                         else:
                             last_exception = last_exception or (
-                                "No Response received from the remote unit"
+                                "No Response received from the remote slave"
                                 "/Unable to decode response"
                             )
                             response = ModbusIOException(
                                 last_exception, request.function_code
                             )
-                        if self.reset_socket:
-                            self.client.close()
+                        self.client.close()
                     if hasattr(self.client, "state"):
-                        _logger.debug(
+                        Log.debug(
                             "Changing transaction state from "
                             '"PROCESSING REPLY" to '
                             '"TRANSACTION_COMPLETE"'
@@ -241,25 +245,20 @@ class ModbusTransactionManager:
                 return response
             except ModbusIOException as exc:
                 # Handle decode errors in processIncomingPacket method
-                _logger.exception(exc)
+                Log.error("Modbus IO exception {}", exc)
                 self.client.state = ModbusTransactionState.TRANSACTION_COMPLETE
-                if self.reset_socket:
-                    self.client.close()
+                self.client.close()
                 return exc
 
     def _retry_transaction(self, retries, reason, packet, response_length, full=False):
         """Retry transaction."""
-        txt = f"Retry on {reason} response - {retries}"
-        _logger.debug(txt)
-        _logger.debug(
-            'Changing transaction state from "WAITING_FOR_REPLY" to "RETRYING"'
-        )
+        Log.debug("Retry on {} response - {}", reason, retries)
+        Log.debug('Changing transaction state from "WAITING_FOR_REPLY" to "RETRYING"')
         self.client.state = ModbusTransactionState.RETRYING
         if self.backoff:
             delay = 2 ** (self.retries - retries) * self.backoff
             time.sleep(delay)
-            txt = f"Sleeping {delay}"
-            _logger.debug(txt)
+            Log.debug("Sleeping {}", delay)
         self.client.connect()
         if hasattr(self.client, "_in_waiting"):
             if (
@@ -270,9 +269,7 @@ class ModbusTransactionManager:
                     return result, None
         return self._transact(packet, response_length, full=full)
 
-    def _transact(
-        self, packet, response_length, full=False, broadcast=False
-    ):  # pylint: disable=too-complex
+    def _transact(self, packet, response_length, full=False, broadcast=False):
         """Do a Write and Read transaction.
 
         :param packet: packet to be sent
@@ -286,64 +283,50 @@ class ModbusTransactionManager:
         try:
             self.client.connect()
             packet = self.client.framer.buildPacket(packet)
-            if _logger.isEnabledFor(logging.DEBUG):
-                txt = f"SEND: {hexlify_packets(packet)}"
-                _logger.debug(txt)
+            Log.debug("SEND: {}", packet, ":hex")
             size = self._send(packet)
             if (
                 isinstance(size, bytes)
                 and self.client.state == ModbusTransactionState.RETRYING
             ):
-                _logger.debug(
+                Log.debug(
                     "Changing transaction state from "
                     '"RETRYING" to "PROCESSING REPLY"'
                 )
                 self.client.state = ModbusTransactionState.PROCESSING_REPLY
                 return size, None
+            if self.client.comm_params.handle_local_echo is True:
+                if self._recv(size, full) != packet:
+                    return b"", "Wrong local echo"
             if broadcast:
                 if size:
-                    _logger.debug(
+                    Log.debug(
                         'Changing transaction state from "SENDING" '
                         'to "TRANSACTION_COMPLETE"'
                     )
                     self.client.state = ModbusTransactionState.TRANSACTION_COMPLETE
                 return b"", None
             if size:
-                _logger.debug(
+                Log.debug(
                     'Changing transaction state from "SENDING" '
                     'to "WAITING FOR REPLY"'
                 )
                 self.client.state = ModbusTransactionState.WAITING_FOR_REPLY
-            if (
-                hasattr(self.client, "handle_local_echo")
-                and self.client.handle_local_echo is True
-            ):
-                if self._recv(size, full) != packet:
-                    return b"", "Wrong local echo"
             result = self._recv(response_length, full)
             # result2 = self._recv(response_length, full)
-            if _logger.isEnabledFor(logging.DEBUG):
-                txt = f"RECV: {hexlify_packets(result)}"
-                _logger.debug(txt)
-
-        except (
-            socket.error,
-            ModbusIOException,
-            InvalidMessageReceivedException,
-        ) as msg:
-            if self.reset_socket:
-                self.client.close()
-            txt = f"Transaction failed. ({msg}) "
-            _logger.debug(txt)
+            Log.debug("RECV: {}", result, ":hex")
+        except (OSError, ModbusIOException, InvalidMessageReceivedException) as msg:
+            self.client.close()
+            Log.debug("Transaction failed. ({}) ", msg)
             last_exception = msg
             result = b""
         return result, last_exception
 
-    def _send(self, packet, retrying=False):  # pylint: disable=unused-argument
+    def _send(self, packet, _retrying=False):
         """Send."""
         return self.client.framer.sendPacket(packet)
 
-    def _recv(self, expected_response_length, full):  # pylint: disable=too-complex
+    def _recv(self, expected_response_length, full):  # noqa: C901
         """Receive."""
         total = None
         if not full:
@@ -389,15 +372,14 @@ class ModbusTransactionManager:
                     elif expected_response_length is None and isinstance(
                         self.client.framer, ModbusRtuFramer
                     ):
-                        try:
+                        with suppress(
+                            IndexError  # response length indeterminate with available bytes
+                        ):
                             expected_response_length = (
                                 self.client.framer.get_expected_response_length(
                                     read_min
                                 )
                             )
-                        except IndexError:
-                            # Could not determine response length with available bytes
-                            pass
                     if expected_response_length is not None:
                         expected_response_length -= min_size
                         total = expected_response_length + min_size
@@ -414,25 +396,25 @@ class ModbusTransactionManager:
         actual = len(result)
         if total is not None and actual != total:
             msg_start = "Incomplete message" if actual else "No response"
-            txt = (
-                "{msg_start} received, "
-                f"Expected {total} bytes Received "
-                f"{actual} bytes !!!!"
+            Log.debug(
+                "{} received, Expected {} bytes Received {} bytes !!!!",
+                msg_start,
+                total,
+                actual,
             )
-            _logger.debug(txt)
         elif not actual:
             # If actual == 0 and total is not None then the above
             # should be triggered, so total must be None here
-            _logger.debug("No response received to unbounded read !!!!")
+            Log.debug("No response received to unbounded read !!!!")
         if self.client.state != ModbusTransactionState.PROCESSING_REPLY:
-            _logger.debug(
+            Log.debug(
                 "Changing transaction state from "
                 '"WAITING FOR REPLY" to "PROCESSING REPLY"'
             )
             self.client.state = ModbusTransactionState.PROCESSING_REPLY
         return result
 
-    def addTransaction(self, request, tid=None):  # pylint: disable=invalid-name
+    def addTransaction(self, request, tid=None):
         """Add a transaction to the handler.
 
         This holds the request in case it needs to be resent.
@@ -444,7 +426,7 @@ class ModbusTransactionManager:
         """
         raise NotImplementedException("addTransaction")
 
-    def getTransaction(self, tid):  # pylint: disable=invalid-name
+    def getTransaction(self, tid):
         """Return a transaction matching the referenced tid.
 
         If the transaction does not exist, None is returned
@@ -454,7 +436,7 @@ class ModbusTransactionManager:
         """
         raise NotImplementedException("getTransaction")
 
-    def delTransaction(self, tid):  # pylint: disable=invalid-name
+    def delTransaction(self, tid):
         """Remove a transaction matching the referenced tid.
 
         :param tid: The transaction to remove
@@ -462,7 +444,7 @@ class ModbusTransactionManager:
         """
         raise NotImplementedException("delTransaction")
 
-    def getNextTID(self):  # pylint: disable=invalid-name
+    def getNextTID(self):
         """Retrieve the next unique transaction identifier.
 
         This handles incrementing the identifier after
@@ -475,7 +457,7 @@ class ModbusTransactionManager:
 
     def reset(self):
         """Reset the transaction identifier."""
-        self.tid = Defaults.TransactionId
+        self.tid = 0
         self.transactions = type(  # pylint: disable=attribute-defined-outside-init
             self.transactions
         )()
@@ -505,15 +487,14 @@ class DictTransactionManager(ModbusTransactionManager):
     def addTransaction(self, request, tid=None):
         """Add a transaction to the handler.
 
-        This holds the requets in case it needs to be resent.
+        This holds the requests in case it needs to be resent.
         After being sent, the request is removed.
 
         :param request: The request to hold on to
         :param tid: The overloaded transaction id to use
         """
         tid = tid if tid is not None else request.transaction_id
-        txt = f"Adding transaction {tid}"
-        _logger.debug(txt)
+        Log.debug("Adding transaction {}", tid)
         self.transactions[tid] = request
 
     def getTransaction(self, tid):
@@ -524,8 +505,7 @@ class DictTransactionManager(ModbusTransactionManager):
         :param tid: The transaction to retrieve
 
         """
-        txt = f"Getting transaction {tid}"
-        _logger.debug(txt)
+        Log.debug("Getting transaction {}", tid)
         if not tid:
             if self.transactions:
                 return self.transactions.popitem()[1]
@@ -537,79 +517,5 @@ class DictTransactionManager(ModbusTransactionManager):
 
         :param tid: The transaction to remove
         """
-        txt = f"deleting transaction {tid}"
-        _logger.debug(txt)
-
+        Log.debug("deleting transaction {}", tid)
         self.transactions.pop(tid, None)
-
-
-class FifoTransactionManager(ModbusTransactionManager):
-    """Implements a transaction.
-
-    For a manager where the results are returned in a FIFO manner.
-    """
-
-    def __init__(self, client, **kwargs):
-        """Initialize an instance of the ModbusTransactionManager.
-
-        :param client: The client socket wrapper
-        """
-        super().__init__(client, **kwargs)
-        self.transactions = []
-
-    def __iter__(self):
-        """Iterate over the current managed transactions.
-
-        :returns: An iterator of the managed transactions
-        """
-        return iter(self.transactions)
-
-    def addTransaction(self, request, tid=None):
-        """Add a transaction to the handler.
-
-        This holds the requets in case it needs to be resent.
-        After being sent, the request is removed.
-
-        :param request: The request to hold on to
-        :param tid: The overloaded transaction id to use
-        """
-        tid = tid if tid is not None else request.transaction_id
-        txt = f"Adding transaction {tid}"
-        _logger.debug(txt)
-
-        self.transactions.append(request)
-
-    def getTransaction(self, tid):
-        """Return a transaction matching the referenced tid.
-
-        If the transaction does not exist, None is returned
-
-        :param tid: The transaction to retrieve
-        """
-        return self.transactions.pop(0) if self.transactions else None
-
-    def delTransaction(self, tid):
-        """Remove a transaction matching the referenced tid.
-
-        :param tid: The transaction to remove
-        """
-        txt = f"Deleting transaction {tid}"
-        _logger.debug(txt)
-        if self.transactions:
-            self.transactions.pop(0)
-
-
-# --------------------------------------------------------------------------- #
-# Exported symbols
-# --------------------------------------------------------------------------- #
-
-
-__all__ = [
-    "FifoTransactionManager",
-    "DictTransactionManager",
-    "ModbusSocketFramer",
-    "ModbusTlsFramer",
-    "ModbusRtuFramer",
-    "ModbusAsciiFramer",
-    "ModbusBinaryFramer",
-]
