@@ -3,6 +3,20 @@
 A collection of utilities for packing data, unpacking
 data computing checksums, and decode checksums.
 """
+from __future__ import annotations
+
+
+__all__ = [
+    "pack_bitstring",
+    "unpack_bitstring",
+    "default",
+    "computeCRC",
+    "checkCRC",
+    "computeLRC",
+    "checkLRC",
+    "rtuFrameSize",
+]
+
 # pylint: disable=missing-type-doc
 import struct
 
@@ -64,27 +78,21 @@ def dict_property(store, index):
         getter = lambda self: store(  # pylint: disable=unnecessary-lambda-assignment
             self
         )[index]
-        setter = lambda self, value: store(  # pylint: disable=unnecessary-dunder-call,unnecessary-lambda-assignment
+        setter = lambda self, value: store(  # pylint: disable=unnecessary-lambda-assignment
             self
-        ).__setitem__(
-            index, value
-        )
+        ).__setitem__(index, value)
     elif isinstance(store, str):
         getter = lambda self: self.__getattribute__(  # pylint: disable=unnecessary-dunder-call,unnecessary-lambda-assignment
             store
-        )[
-            index
-        ]
+        )[index]
         setter = lambda self, value: self.__getattribute__(  # pylint: disable=unnecessary-dunder-call,unnecessary-lambda-assignment
             store
-        ).__setitem__(
-            index, value
-        )
+        ).__setitem__(index, value)
     else:
-        getter = lambda self: store[  # pylint: disable=unnecessary-lambda-assignment
-            index
-        ]
-        setter = lambda self, value: store.__setitem__(  # pylint: disable=unnecessary-dunder-call,unnecessary-lambda-assignment
+        getter = (
+            lambda self: store[index]  # pylint: disable=unnecessary-lambda-assignment
+        )
+        setter = lambda self, value: store.__setitem__(  # pylint: disable=unnecessary-lambda-assignment
             index, value
         )
 
@@ -94,10 +102,10 @@ def dict_property(store, index):
 # --------------------------------------------------------------------------- #
 # Bit packing functions
 # --------------------------------------------------------------------------- #
-def pack_bitstring(bits):
-    """Create a string out of an array of bits.
+def pack_bitstring(bits: list[bool]) -> bytes:
+    """Create a bytestring out of a list of bits.
 
-    :param bits: A bit array
+    :param bits: A list of bits
 
     example::
 
@@ -121,37 +129,26 @@ def pack_bitstring(bits):
     return ret
 
 
-def unpack_bitstring(string):
-    """Create bit array out of a string.
+def unpack_bitstring(data: bytes) -> list[bool]:
+    """Create bit list out of a bytestring.
 
-    :param string: The modbus data packet to decode
+    :param data: The modbus data packet to decode
 
     example::
 
         bytes  = "bytes to decode"
         result = unpack_bitstring(bytes)
     """
-    byte_count = len(string)
+    byte_count = len(data)
     bits = []
     for byte in range(byte_count):
-        value = int(int(string[byte]))
+        value = int(int(data[byte]))
         for _ in range(8):
             bits.append((value & 1) == 1)
             value >>= 1
     return bits
 
 
-def make_byte_string(byte_string):
-    """Return byte string from a given string, python3 specific fix.
-
-    :param byte_string:
-    :return:
-    """
-    if isinstance(byte_string, str):
-        byte_string = byte_string.encode()
-    return byte_string
-
-
 # --------------------------------------------------------------------------- #
 # Error Detection Functions
 # --------------------------------------------------------------------------- #
@@ -184,7 +181,7 @@ def computeCRC(data):  # pylint: disable=invalid-name
     For modbus, this is only used on the binary serial protocols (in this
     case RTU).
 
-    The difference between modbus"s crc16 and a normal crc16
+    The difference between modbus's crc16 and a normal crc16
     is that modbus starts the crc value out at 0xffff.
 
     :param data: The data to create a crc16 of
@@ -213,7 +210,7 @@ def computeLRC(data):  # pylint: disable=invalid-name
 
     This is only used on the serial ASCII
     modbus protocol. A full description of this implementation
-    can be found in appendex B of the serial line modbus description.
+    can be found in appendix B of the serial line modbus description.
 
     :param data: The data to apply a lrc to
     :returns: The calculated LRC
@@ -266,18 +263,3 @@ def hexlify_packets(packet):
     if not packet:
         return ""
     return " ".join([hex(int(x)) for x in packet])
-
-
-# --------------------------------------------------------------------------- #
-# Exported symbols
-# --------------------------------------------------------------------------- #
-__all__ = [
-    "pack_bitstring",
-    "unpack_bitstring",
-    "default",
-    "computeCRC",
-    "checkCRC",
-    "computeLRC",
-    "checkLRC",
-    "rtuFrameSize",
-]
